<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * DB_Complex administration class
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 *  Require the parent class definition
 */
require_once('LiveUser/Perm/Container/DB_Complex.php'); 

/**
 * Class LiveUser_Perm_Container_DB_Complex_Admin
 * 
 * Description:
 * This is a PEAR::DB admin class for the LiveUser package.
 * A PEAR::DB connection object can be passed to the constructor to reuse an
 * existing connection. Alternatively, a DSN can be passed to open a new one.
 * 
 * Requirements:
 * - Files "common.php", "Container/DB_Complex.php" in directory "Perm"
 * - Array of connection options must be passed to the constructor.
 *   Example: array("server" => "localhost", "user" => "root",
 *   "password" => "pwd", "database" => "AllMyPreciousData")
 *  
 * @author  Christian Dickmann <dickmann@php.net>
 * @author  Markus Wolff <wolff@21st.de>
 * @version $Id: DB_Complex_Admin.php,v 1.1 2003/05/22 00:06:39 krausbn Exp $
 * @package LiveUser
 * @category authentication
 */ 
class LiveUser_Perm_Container_DB_Complex_Admin extends LiveUser_Perm_Container_DB_Complex
{
	/**
	 * Associative array of all available languages (short name (i.e. "DE", "EN"...) as key, database id as value)
	 * @access private
	 */
	var $_langs;
    
    /**
     * Adds a Group to the Database
     *
     * @param array   $names       names of new group
     * @param array   $comments    (optional) descriptions of new group
     * @param string  $owner_user  (optional) owner user id of new group
     * @param integer $owner_group (optional) owner group id of new group
     * @param boolean $active      (optional) activate group?
     *
     * @access public
     * @return integer id of new group
     */
    
    //function addGroup($names, $comments = array(), $owner_user = NULL, $owner_group = NULL, $active = false)
    function addGroup($define_name, $active = false, $owner_user = NULL, $owner_group = NULL)
    {
        
        // Get next group ID
        $group_id = $this->dbc->nextId('liveuser_groups');
        if (DB::isError($group_id)) {
            return $group_id;
        };
        
        // Insert Group into Groupstable
        $sql = 'INSERT INTO 
                  liveuser_groups
                  (group_id, group_define_name, owner_user_id, owner_group_id, is_active)
                VALUES
                  (
                    '.(int)$group_id.',
                    '.$this->dbc->quote($define_name).', 
                    '.(($owner_user  === NULL) ? 'NULL' : $this->dbc->quote($owner_user)).',
                    '.(($owner_group === NULL) ? 'NULL' : (int)$owner_group).',
                    "'.($active ? 'Y' : 'N').'"
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return $group_id;
    }
    
    /**
     * Assigns name (and comment) in specified language to a group 
     *
     * @param integer $group_id      id of group
     * @param string  $language      language (two letter code) of name/comment
     * @param string  $group_name    name of group
     * @param string  $group_comment (optional) description of group
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    
    function addGroupName($group_id, $language, $group_name, $group_comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Register group name
        $sql = 'INSERT INTO 
                  liveuser_group_names
                  (group_id, language_id, group_name, group_comment)
                VALUES
                  (
                    '.(int)$group_id.',
                    '.(int)$this->_langs[$language].',
                    '.$this->dbc->quote($group_name).',
                    '.(($group_comment === NULL) ? 'NULL' : $this->dbc->quote($group_comment)).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Group name (and comment) added ...
        return true;
    }
    
    /**
     * Updates name (and comment) of group in specified language 
     *
     * @param integer $group_id      id of group
     * @param string  $language      language (two letter code) of name/comment
     * @param string  $group_name    name of group
     * @param string  $group_comment (optional) description of group
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
     
    function updateGroupName($group_id, $language, $group_name, $group_comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Register new group name
        $sql = 'UPDATE
                  liveuser_group_names
                SET
                  group_name    = '.$this->dbc->quote($group_name).',
                  group_comment = '.(($group_comment === NULL) ? 'NULL' : $this->dbc->quote($group_comment)).'
                WHERE
                  group_id    = '.(int)$group_id.' AND
                  language_id = '.(int)$this->_langs[$language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Group name (and comment) updated ...
        return true;
    }
    
    /**
     * Remove name (and comment) of the group in specified language 
     *
     * @param integer $group_id id of group
     * @param string  $language language (two letter code) of name/comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
     
    function removeGroupName($group_id, $language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Remove group name
        $sql = 'DELETE FROM 
                  liveuser_group_names
                WHERE 
                  group_id    = '.(int)$group_id.' AND 
                  language_id = '.(int)$this->_langs[$language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Group name (and comment) removed ...
        return true;
    }
    
    /**
     * Reads all languages from databases and stores them in private variable
     *
     * $this->_langs is filled with this array structure:
     *     two_letter_code => language_id
     *
     * @access private
     * @return mixed boolean or DB Error object
     **/
     
    function _getLanguages()
    {
        if (!isset($this->_langs)) {
            $this->_langs = $this->dbc->getAssoc('SELECT two_letter_name, language_id FROM liveuser_languages');
            if (DB::isError($this->_langs)) {
                return $this->_langs;
            };
        };
        return true;
    }
    
    /**
     * assign subgroup to parent group
     *
     * @param integer $group_id     id of parent group
     * @param integer $subgroup_id  id of child group
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    
    function assignSubgroup($group_id, $subgroup_id)
    {
        // XXX test for infinite loop
        $sql = 'INSERT INTO
                  liveuser_group_subgroups
                  (group_id, subgroup_id)
                VALUES
                  (
                    '.(int)$group_id.',
                    '.(int)$subgroup_id.'
                  )';
        $result = $this->dbc->query($sql);
        
        return $result;
    }
    
    /**
     * Deletes a group from the database
     *
     * @param integer $group_id  id of deleted group
     * @param boolean $recursive (optional) recursive delete of all subgroups
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    
    function removeGroup($group_id, $recursive = false)
    {
        // Recursive delete groups
        if ($recursive) {
            // Get all subgroups
            $sql = 'SELECT 
                      subgroup_id 
                    FROM 
                      liveuser_group_subgroups
                    WHERE
                      group_id = '.(int)$group_id;
            $result = $this->dbc->getCol($sql);
            if (DB::isError($result)) {
                return $result;
            };
            
            // Recursive removeGroup() call for every subgroup
            foreach ($result as $subgroup_id) {
                $res = $this->removeGroup($subgroup_id, true);
                if (DB::isError($res)) {
                    return $res;
                };
            };
        };
        
        // Delete group itself
        $sql = 'DELETE FROM
                  liveuser_groups
                WHERE 
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete Subgroup assignments
        $sql = 'DELETE FROM 
                  liveuser_group_subgroups
                WHERE
                  group_id = '.(int)$group_id.' OR
                  subgroup_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete group names/comments
        $sql = 'DELETE FROM 
                  liveuser_group_names
                WHERE
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
           
        // Delete user assignments
        $sql = 'DELETE FROM 
                  liveuser_groupusers
                WHERE
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete group rights 
        $sql = 'DELETE FROM 
                  liveuser_grouprights
                WHERE
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return true;
    }
    
    /**
     * Update group 
     *
     * @param integer $group_id    group id
     * @param boolean $active      (optional) activate group?
     * @param string  $owner_user  (optional) owner user id of group
     * @param integer $owner_group (optional) owner group id of group
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateGroup($group_id, $active = false, $owner_user = NULL, $owner_group = NULL)
    {
        $sql = 'UPDATE
                  liveuser_groups
                SET
                  owner_user_id  = '.(($owner_user  !== NULL) ? $this->dbc->quote($owner_user) : 'NULL').',
                  owner_group_id = '.(($owner_group !== NULL) ? (int)$owner_group : 'NULL').',
                  is_active      = '.($active ? '"Y"' : '"N"').'
                WHERE 
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return true;
    }
    
    /**
     * Activate group 
     *
     * @param integer $group_id group id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function activateGroup($group_id)
    {
        $sql = 'UPDATE
                  liveuser_groups
                SET
                  is_active = "Y"
                WHERE 
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return true;
    }
    
    /**
     * Deactivate group 
     *
     * @param integer $group_id group id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function deactivateGroup($group_id)
    {
        $sql = 'UPDATE
                  liveuser_groups
                SET
                  is_active = "N"
                WHERE 
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return true;
    }
    
    /**
     * Add a new language
     *
     * @param string $two_letter_code two letter code of language
     * @param string $native_name     native name of language
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function addLanguage($two_letter_code, $native_name)
    {
        // Get next language id
        $language_id = $this->dbc->nextId('liveuser_languages');
        if (DB::isError($language_id)) {
            return $language_id;
        };
        
        // Add language
        $sql = 'INSERT INTO 
                  liveuser_languages
                  (language_id, two_letter_name, native_name)
                VALUES
                  (
                    '.$language_id.',
                    '.$this->dbc->quote($two_letter_code).',
                    '.$this->dbc->quote($native_name).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Clear language cache
        unset($this->_langs);
        
        // Job done ...
        return $language_id;
    }
    
    /**
     * Remove a language
     *
     * @param integer $language language (two letter code)
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeLanguage($language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Delete language
        $sql = 'DELETE FROM 
                  liveuser_languages
                WHERE
                  language_id = '.(int)$this->_langs[$language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete language names
        $sql = 'DELETE FROM 
                  liveuser_language_names
                WHERE
                  language_id = '.(int)$language_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Clear language cache
        unset($this->_langs);
        
        // Job done ...
        return true;
    }
    
    /**
     * Update language
     *
     * @param integer $language        language (two letter code), which is going to be edited
     * @param string  $native_name     new native name of language
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateLanguage($language, $native_name)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'UPDATE
                  liveuser_languages
                SET
                  native_name     = '.$this->dbc->quote($native_name).'
                WHERE
                  language_id     = '.(int)$this->_langs[$language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
           
        // Clear language cache
        unset($this->_langs);
        
        // Job done ...
        return true;
    }
    
    /**
     * Add language translation
     *
     * @param integer $language             language (two letter code), which is going to be translated
     * @param integer $translation_language language (two letter code) of new translation 
     * @param string  $name                 translation of language name
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function addLanguageName($language, $translation_language, $name)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language]) || !isset($this->_langs[$translation_language])) {
            return false;
        };
        
        $sql = 'INSERT INTO 
                  liveuser_language_names
                  (native_language_id, language_id, native_name)
                VALUES
                  (
                    '.(int)$this->_langs[$language].',
                    '.(int)$this->_langs[$translation_language].',
                    '.$this->dbc->quote($name).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    /**
     * Delete language translation
     *
     * @param integer $language             language (two letter code), which is translated
     * @param integer $translation_language language (two letter code) of translation 
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeLanguageName($language, $translation_language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language]) || !isset($this->_langs[$translation_language])) {
            return false;
        };
        
        $sql = 'DELETE FROM
                  liveuser_language_names
                WHERE
                  native_language_id = '.(int)$this->_langs[$language].' AND
                  language_id        = '.(int)$this->_langs[$translation_language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    /**
     * Update language translation
     *
     * @param integer $language             language (two letter code), which is translated
     * @param integer $translation_language language (two letter code) of translation 
     * @param string  $name                 translation of language name
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateLanguageName($language, $translation_language, $name)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language]) || !isset($this->_langs[$translation_language])) {
            return false;
        };
        
        $sql = 'UPDATE
                  liveuser_language_names
                SET
                  native_name = '.$this->dbc->quote($name).'
                WHERE
                  native_language_id = '.(int)$this->_langs[$language].' AND
                  language_id        = '.(int)$this->_langs[$translation_language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }

    
        
    /**
     * Add an application
     *
     * @param string $define_name name of application constant. thats the key 
     *                            to access the application in scripts.
     *
     * @access public
     * @return mixed integer (application id) or DB Error object
     **/
    function addApplication($define_name)
    {
        // Get next application id
        $application_id = $this->dbc->nextId('liveuser_applications');
        if (DB::isError($application_id)) {
            return $application_id;
        };
        
        // Register new application
        $sql = 'INSERT INTO 
                  liveuser_applications
                  (application_id, application_define_name)
                VALUES
                  (
                    '.(int)$application_id.',
                    '.$this->dbc->quote($define_name).'
                  )';
        $result = $this->dbc->query($sql);
        
        if (DB::isError($result)) {
            return $result;
        };
                 
        return $application_id;
    }

    
    
    
    /**
     * Add an area
     *
     * @param string $define_name name of area constant. thats the key 
     *                            to access the area in scripts.
     *
     * @access public
     * @return mixed integer (area id) or DB Error object
     **/
    function addArea($application_id, $define_name)
    {
        // Get next area id
        $area_id = $this->dbc->nextId('liveuser_areas');
        if (DB::isError($area_id)) {
            return $area_id;
        };
        
        // Register new area
        $sql = 'INSERT INTO 
                  liveuser_areas
                  (area_id, area_define_name, application_id)
                VALUES
                  (
                    '.(int)$area_id.',
                    '.$this->dbc->quote($define_name).',
        		    '.(int)$application_id.'
                  )';
        $result = $this->dbc->query($sql);
        
        if (DB::isError($result)) {
            return $result;
        };
                 
        return $area_id;
    }
    
    /**
     * Delete an application
     *
     * @param integer $application_id	application (id) to be deleted
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeApplication($application_id)
    {   
    	// Get all areas within the application, no matter what language
        $areas = $this->dbc->getAll("SELECT area_id FROM liveuser_areas WHERE application_id=$application_id",
                                    NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($areas)) {
        	return $areas;
        }
        
        // Delete all areas within the application
        foreach($areas as $area) {
        	$res = $this->removeArea($area['area_id']);
        	if (DB::isError($res)) {
        		return $res;
        	}
        }
    	
        // Delete application names/comments
        $sql = 'DELETE FROM
                  liveuser_application_names
                WHERE
                  application_id = '.(int)$application_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };

        
    	// Delete application itself
        $sql = 'DELETE FROM
                  liveuser_applications
                WHERE
                  application_id = '.(int)$application_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                
        return true;
    }
    
    
    /**
     * Delete an area
     *
     * @param integer $area_id area (id), which is deleted
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeArea($area_id)
    {   
        // Delete area itself
        $sql = 'DELETE FROM
                  liveuser_areas
                WHERE
                  area_id = '.(int)$area_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete area names/comments
        $sql = 'DELETE FROM
                  liveuser_area_names
                WHERE
                  area_id = '.(int)$area_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete all rights in this area
        $sql = 'SELECT
                  right_id
                FROM
                  liveuser_rights
                WHERE
                  area_id = '.(int)$area_id;
        $result = $this->dbc->getCol($sql);
        if (DB::isError($result)) {
            return $result;
        };
        if (is_array($result)) {
            foreach($result as $right_id) {
                $this->removeRight($right_id);
            };
        };
        
        // Delete area admins --- XXX
        
        return true;
    }
    
    /**
     * Update an application
     *
     * @param integer $application_id     application (id), which is updated
     * @param string  $define_name name of application constant. thats the key 
     *                             to access the application in scripts.
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateApplication($application_id, $define_name)
    {
        $sql = 'UPDATE 
                  liveuser_applications
                SET
                  application_define_name = '.$this->dbc->quote($define_name).'
                WHERE
                  application_id = '.(int)$application_id;
        
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    
    
    /**
     * Update an area
     *
     * @param integer $area_id     area (id), which is updated
     * @param string  $define_name name of area constant. thats the key 
     *                             to access the area in scripts.
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateArea($area_id, $define_name)
    {
        $sql = 'UPDATE 
                  liveuser_areas
                SET
                  area_define_name = '.$this->dbc->quote($define_name).'
                WHERE
                  area_id = '.(int)$area_id;
        
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }

        
    /**
     * Add a application name
     *
     * @param integer $application_id  application id
     * @param string  $language language of new name (two letter code)
     * @param string  $name     new translation of name
     * @param string  $comment  (optional) new translation of comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function addApplicationName($application_id, $language, $name, $comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Register new application name
        $sql = 'INSERT INTO 
                  liveuser_application_names
                  (application_id, language_id, application_name, application_comment)
                VALUES
                  (
                    '.(int)$application_id.',
                    '.(int)($this->_langs[$language]).',
                    '.$this->dbc->quote($name).',
                    '.(($comment === NULL) ? 'NULL' : $this->dbc->quote($comment)).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return true;
    }

    
    
    
    
    /**
     * Add a area name
     *
     * @param integer $area_id  area id
     * @param string  $language language of new name (two letter code)
     * @param string  $name     new translation of name
     * @param string  $comment  (optional) new translation of comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function addAreaName($area_id, $language, $name, $comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Register new area name
        $sql = 'INSERT INTO 
                  liveuser_area_names
                  (area_id, language_id, area_name, area_comment)
                VALUES
                  (
                    '.(int)$area_id.',
                    '.(int)($this->_langs[$language]).',
                    '.$this->dbc->quote($name).',
                    '.(($comment === NULL) ? 'NULL' : $this->dbc->quote($comment)).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        return true;
    }
    
    
    /**
     * Delete a application name
     *
     * @param integer $application_id  application id
     * @param string  $language language of application name (two letter code)
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeApplicationName($application_id, $language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Remove application name
        $sql = 'DELETE FROM 
                  liveuser_application_names
                WHERE 
                  application_id = '.(int)$application_id.' AND
                  language_id = '.(int)$this->_langs[$language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    
    
    
    /**
     * Delete a area name
     *
     * @param integer $area_id  area id
     * @param string  $language language of area name (two letter code)
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeAreaName($area_id, $language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Remove area name
        $sql = 'DELETE FROM 
                  liveuser_area_names
                WHERE 
                  area_id = '.(int)$area_id.' AND
                  language_id = '.(int)$this->_langs[$language];
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    /**
     * Update a application name
     *
     * @param integer $application_id  application id
     * @param string  $language language of name (two letter code)
     * @param string  $name     new translation of name
     * @param string  $comment  (optional) new translation of comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateApplicationName($application_id, $language, $name, $comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Update application name
        $sql = 'UPDATE 
                  liveuser_application_names 
                SET
                  application_name    = '.$this->dbc->quote($name).',
                  application_comment = '.(($comment === NULL) ? 'NULL' : $this->dbc->quote($comment)).'
                WHERE
                  application_id = '.(int)$application_id.' AND
                  language_id = '.(int)($this->_langs[$language]);
        $result = $this->dbc->query($sql);
        
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    
    
    /**
     * Update a area name
     *
     * @param integer $area_id  area id
     * @param string  $language language of name (two letter code)
     * @param string  $name     new translation of name
     * @param string  $comment  (optional) new translation of comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateAreaName($area_id, $language, $name, $comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Update area name
        $sql = 'UPDATE 
                  liveuser_area_names 
                SET
                  area_name    = '.$this->dbc->quote($name).',
                  area_comment = '.(($comment === NULL) ? 'NULL' : $this->dbc->quote($comment)).'
                WHERE
                  area_id = '.(int)$area_id.' AND
                  language_id = '.(int)($this->_langs[$language]);
        $result = $this->dbc->query($sql);
        
        if (DB::isError($result)) {
            return $result;
        };
                 
        return true;
    }
    
    /**
     * Add a right in special area
     *
     * @param integer $area_id     area id
     * @param string  $define_name name of constant to access this 
     *                             right in scripts
     * @param boolean $has_level   apply levels to this right?
     *
     * @access public
     * @return mixed integer (area_id) or DB Error object
     **/
    function addRight($area_id, $define_name, $has_level = false)
    {
        // Get next right id
        $right_id = $this->dbc->nextId('liveuser_rights');
        if (DB::isError($right_id)) {
            return $right_id;
        };
        
        // Register right
        $sql = 'INSERT INTO 
                  liveuser_rights
                  (right_id, area_id, right_define_name, has_level)
                VALUES
                  ( 
                    '.(int)$right_id.',
                    '.(int)$area_id.',
                    '.$this->dbc->quote($define_name).',
                    '.$this->dbc->quote(($has_level   ? 'Y' : 'N')).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        // Job done ...
        return $right_id;
    }
    
    /**
     * Delete a right
     *
     * @param integer $right_id right id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeRight($right_id)
    {
        // Delete right itself
        $sql = 'DELETE FROM
                  liveuser_rights
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete userright
        $sql = 'DELETE FROM
                  liveuser_userrights
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete group right
        $sql = 'DELETE FROM
                  liveuser_grouprights
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete right names/comments
        $sql = 'DELETE FROM
                  liveuser_right_names
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete right from imply-table
        $sql = 'DELETE FROM
                  liveuser_right_implied
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete right scopes
        $sql = 'DELETE FROM
                  liveuser_right_scope
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
       
    }
    
    /**
     * Update a right
     *
     * @param integer $right_id    right id
     * @param integer $area_id     area id
     * @param string  $define_name name of constant to access this 
     *                             right in scripts
     * @param boolean $has_level   apply levels to this right?
     *
     * @access public
     * @return mixed boolean or DB Error object
     */
    function updateRight($right_id, $area_id, $define_name, $has_level = false)
    {
        $sql = 'UPDATE
                  liveuser_rights
                SET
                  area_id           = '.(int)$area_id.',
                  right_define_name = '.$this->dbc->quote($define_name).',
                  has_level         = '.$this->dbc->quote(($has_level   ? 'Y' : 'N')).'
                WHERE
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Add a right name
     *
     * @param integer $right_id  right id
     * @param string  $language  language of translation (two letter code)
     * @param string  $name      new translation of right name
     * @param string  $comment   (optional) new translation of comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function addRightName($right_id, $language, $name, $comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Register new right name
        $sql = 'INSERT INTO 
                  liveuser_right_names
                  (right_id, language_id, right_name, right_comment)
                VALUES
                  (
                    '.(int)$right_id.',
                    '.(int)($this->_langs[$language]).',
                    '.$this->dbc->quote($name).',
                    '.(($comment === NULL) ? 'NULL' : $this->dbc->quote($comment)).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Delete a right name
     *
     * @param integer $right_id  right id
     * @param string  $language  language of translation (two letter code)
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeRightName($right_id, $language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Delete right name
        $sql = 'DELETE FROM 
                  liveuser_right_names
                WHERE
                  right_id    = '.(int)$right_id.' AND
                  language_id = '.(int)($this->_langs[$language]);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Update a right name
     *
     * @param integer $right_id  right id
     * @param string  $language  language of translation (two letter code)
     * @param string  $name      new translation of right name
     * @param string  $comment   (optional) new translation of comment
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateRightName($right_id, $language, $name, $comment = NULL)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
        
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        // Update right name
        $sql = 'UPDATE
                  liveuser_right_names
                SET
                  right_name    = '.$this->dbc->quote($name).',
                  right_comment = '.(($comment === NULL) ? 'NULL' : $this->dbc->quote($comment)).'
                WHERE
                  right_id    = '.(int)$right_id.' AND
                  language_id = '.(int)($this->_langs[$language]);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Imply right
     *
     * @param integer $right_id          right id
     * @param string  $implied_right_id  id of implied right
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function implyRight($right_id, $implied_right_id)
    {
        // Imply Right
        $sql = 'INSERT INTO 
                  liveuser_right_implied
                  (right_id, implied_right_id)
                VALUES
                  (
                    '.(int)$right_id.',
                    '.(int)$implied_right_id.'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        // Update implied status
        $this->_updateImpliedStatus($right_id);
        
        // Job done ...
        return true;
    }
    
    /**
     * Unimply right
     *
     * @param integer $right_id          right id
     * @param string  $implied_right_id  id of implied right
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function unimplyRight($right_id, $implied_right_id)
    {
        // Unimply right
        $sql = 'DELETE FROM 
                  liveuser_right_implied
                WHERE
                  right_id         = '.(int)$right_id.' AND
                  implied_right_id = '.(int)$implied_right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Update implied status
        $this->_updateImpliedStatus($right_id);
        
        // Job done ...
        return true;
    }
    
    /**
     * Update implied status of right
     *
     * @param integer $right_id right id
     *
     * @access private
     * @return mixed boolean or DB Error object
     **/
    function _updateImpliedStatus($right_id)
    {
        // Are there any implied rights?
        $sql = 'SELECT 
                  Count(*)
                FROM
                  liveuser_right_implied
                WHERE 
                  right_id = '.(int)$right_id;
        $count = $this->dbc->getOne($sql);
        if (DB::isError($count)) {
            return $count;
        };
        
        $sql = 'UPDATE 
                  liveuser_rights
                SET
                  has_implied = "'.(((int)$count > 0) ? 'Y' : 'N').'"
                WHERE 
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    // XXX
    // function *Scope
    
    /**
     * Add a user
     *
     * @param string  $handle          user handle
     * @param string  $password        user password
     * @param integer $type            (optional) user type
     * @param string  $owner_user_id   (optional) id of user, who owns the new user
     * @param integer $owner_group_id  (optional) id of group, which owns the new user
     * @param boolean $active          (optional) should this user be activated?
     *
     * @access public
     * @return mixed string (user_id) or DB Error object
     **/
    function addUser($handle, $password, $type = LIVEUSER_USER_TYPE_ID, $owner_user_id = NULL, $owner_group_id = NULL, $active = false)
    {
        // Generate new user id
        $user_id = md5($handle.getmypid().uniqid(rand(),1));
        
        // Encryptt PW? --- XXX
        
        // Register new user
        $sql = 'INSERT INTO
                  liveuser_users
                  (user_id, handle, password, type, owner_user_id, owner_group_id, is_active)
                VALUES
                  (
                    '.$this->dbc->quote($user_id).',
                    '.$this->dbc->quote($handle).',
                    '.$this->dbc->quote($password).',
                    '.(int)$type.',
                    '.$this->dbc->quote($owner_user_id).',
                    '.(int)$owner_group_id.',
                    '.$this->dbc->quote(($active ? 'Y' : 'N')).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return $user_id;
    }
    
    /**
     * Delete user
     *
     * @param string  $user_id  user id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeUser($user_id)
    {
        // Delete user itself
        $sql = 'DELETE FROM 
                  liveuser_users
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete group assignments
        $sql = 'DELETE FROM 
                  liveuser_groupusers
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Delete right assignments
        $sql = 'DELETE FROM 
                  liveuser_userrights
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Update user
     *
     * @param string  $user_id         user id
     * @param string  $handle          new user handle
     * @param string  $password        new user password
     * @param integer $type            (optional) new user type
     * @param string  $owner_user_id   (optional) id of user, who owns the new user
     * @param integer $owner_group_id  (optional) id of group, which owns the new user
     * @param boolean $active          (optional) should this user be activated?
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateUser($user_id, $handle, $password, $type = LIVEUSER_USER_TYPE_ID, $owner_user_id = NULL, $owner_group_id = NULL, $active = false)
    {
        // Encrypt PW? --- XXX
    
        // Update user data
        $sql = 'UPDATE 
                  liveuser_users
                SET
                  handle         = '.$this->dbc->quote($handle).',
                  password       = '.$this->dbc->quote($password).',
                  type           = '.(int)$type.',
                  owner_user_id  = '.$this->dbc->quote($owner_user_id).',
                  owner_group_id = '.(int)$owner_group_id.',
                  is_active      = '.$this->dbc->quote(($active ? 'Y' : 'N')).'
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
                 
        // Job done ...
        return true;
    }
    
    /**
     * Activate user
     *
     * @param string  $user_id  user id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function activateUser($user_id)
    {
        $sql = 'UPDATE
                  liveuser_users
                SET
                  is_active = "Y"
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Deactivate user
     *
     * @param string  $user_id  user id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function deactivateUser($user_id)
    {
        $sql = 'UPDATE
                  liveuser_users
                SET
                  is_active = "N"
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Grand right to user
     *
     * @param string  $user_id     user id
     * @param integer $right_id    right id
     * @param integer $right_level right level
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function grantUserRight($user_id, $right_id, $right_level)
    {
        $sql = 'INSERT INTO 
                  liveuser_userrights
                  (user_id, right_id, right_level)
                VALUES
                  (
                    '.$this->dbc->quote($user_id).',
                    '.(int)$right_id.',
                    '.(int)$right_level.'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Revoke right from user
     *
     * @param string  $user_id     user id
     * @param integer $right_id    right id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function revokeUserRight($user_id, $right_id)
    {
        $sql = 'DELETE FROM 
                  liveuser_userrights
                WHERE
                  user_id = '.$this->dbc->quote($user_id).' AND
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Update right level of userRight
     *
     * @param string  $user_id     user id
     * @param integer $right_id    right id
     * @param integer $right_level right level
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateUserRight($user_id, $right_id, $right_level)
    {
        $sql = 'UPDATE
                  liveuser_userrights
                SET
                  right_level = '.(int)$right_level.'
                WHERE
                  user_id = '.$this->dbc->quote($user_id).' AND
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Grand right to group
     *
     * @param integer $group_id    group id
     * @param integer $right_id    right id
     * @param integer $right_level right level
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function grantGroupRight($group_id, $right_id, $right_level)
    {
        $sql = 'INSERT INTO 
                  liveuser_grouprights
                  (group_id, right_id, right_level)
                VALUES
                  (
                    '.(int)$group_id.',
                    '.(int)$right_id.',
                    '.(int)$right_level.'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Revoke right from group
     *
     * @param integer $group_id    group id
     * @param integer $right_id    right id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function revokeGroupRight($group_id, $right_id)
    {
        $sql = 'DELETE FROM 
                  liveuser_grouprights
                WHERE
                  group_id = '.(int)$group_id.' AND
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Update right level of groupRight
     *
     * @param integer $group_id    group id
     * @param integer $right_id    right id
     * @param integer $right_level right level
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function updateGroupRight($group_id, $right_id, $right_level)
    {
        $sql = 'UPDATE
                  liveuser_grouprights
                SET
                  right_level = '.(int)$right_level.'
                WHERE
                  group_id = '.(int)$group_id.' AND
                  right_id = '.(int)$right_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Add User to Group
     *
     * @param string  $user_id  user id
     * @param integer $group_id group id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function addUserToGroup($user_id, $group_id)
    {
        $sql = 'INSERT INTO 
                  liveuser_groupusers
                  (group_id, user_id)
                VALUES
                  (
                    '.(int)$group_id.',
                    '.$this->dbc->quote($user_id).'
                  )';
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Remove User from Group
     *
     * @param string  $user_id  user id
     * @param integer $group_id group id
     *
     * @access public
     * @return mixed boolean or DB Error object
     **/
    function removeUserFromGroup($user_id, $group_id)
    {
        $sql = 'DELETE FROM
                  liveuser_groupusers
                WHERE
                  user_id  = '.$this->dbc->quote($user_id).' AND
                  group_id = '.(int)$group_id;
        $result = $this->dbc->query($sql);
        if (DB::isError($result)) {
            return $result;
        };
        
        // Job done ...
        return true;
    }
    
    /**
     * Get all granted rights of user
     *
     * Returns array of this structure:
     * array(
     *    right_id => right_level
     * )
     *
     * @param string $user_id user id
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getUserRights($user_id)
    {
        $sql = 'SELECT 
                  right_id,
                  right_level
                FROM 
                  liveuser_userrights
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $userRights = $this->dbc->getAssoc($sql);
        if (DB::isError($userRights)) {
            return $userRights;
        };
        
        return $userRights;
    }
    
    /**
     * Get all groups, the user is a member in
     *
     * Returns array of this structure:
     * array(
     *    group_id
     * )
     *
     * @param string $user_id user id
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getUserGroups($user_id)
    {
        $sql = 'SELECT 
                  group_id
                FROM 
                  liveuser_groupusers
                WHERE 
                  user_id = '.$this->dbc->quote($user_id);
       $groups = $this->dbc->getCol($sql);
       if (DB::isError($groups)) {
           return $groups;
       };
       
       return $groups;
    }
    
    /**
     * Get all granted rights of group
     *
     * Returns array of this structure:
     * array(
     *    right_id => right_level
     * )
     *
     * @param integer $group_id group id
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getGroupRights($group_id)
    {
        $sql = 'SELECT 
                  right_id,
                  right_level
                FROM 
                  liveuser_grouprights
                WHERE
                  group_id = '.(int)$group_id;
        $groupRights = $this->dbc->getAssoc($sql);
        if (DB::isError($groupRights)) {
            return $groupRights;
        };
        
        return $groupRights;
    }
    
    /**
     * Get group info
     *
     * Arraystructure is like this:
     * array(
     *   'name' => 'xyz',
     *   'id' => 123
     *   ...
     * );
     *
     * @param integer $group_id    group id
     * @param string  $language    language (two letter code) of name/comment of group
     * @param boolean $with_rights (optional) list rights of group too?
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getOneGroup($group_id, $language, $with_rights = false)
    {
        // Get all language ids
        if (DB::isError($res = $this->_getLanguages())) {
            return $res;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.group_id as id,
                  main.group_define_name as define_name,
                  names.group_name as name,
                  names.group_comment as comment,
                  main.is_active as is_active
                FROM
                  liveuser_groups as main,
                  liveuser_group_names as names
                WHERE 
                  names.group_id = main.group_id AND 
                  names.language_id = '.(int)$this->_langs[$language].' AND
                  main.group_id = '.(int)$group_id;
                      
        $group = $this->dbc->getRow($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($group)) {
            return $group;
        };
        
        $group['rights'] = $this->getGroupRights($group_id);
        
        return $group;
    }
    
    /**
     * Get list of all groups
     *
     * Only groups are lists, which have a name set in the
     * specified language. Arraystructure is like this:
     * array(
     *   group_id => array(
     *      'name' => 'xyz',
     *      ...
     *   ), ...
     * );
     *
     * @param string  $language    language (two letter code) of name/comment of groups
     * @param boolean $with_rights (optional) list rights of groups too?
     * @param mixed   $parentgroup (optional) do not use this parameter. Its for internal (recursive) use only
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getAllGroups($language, $with_rights = false, $parentgroup = NULL)
    {
        static $groups;
        static $subgroups;
        $result = array();
        
        if ($parentgroup === NULL) {
            // Get all language ids
            if (DB::isError($res = $this->_getLanguages())) {
                return $res;
            };
            
            // Check if language is a known one
            if (!isset($this->_langs[$language])) {
                return false;
            };
            
            $sql = 'SELECT 
                      main.group_id as id,
                      names.group_name as name,
                      names.group_comment as comment,
                      main.is_active as is_active
                    FROM
                      liveuser_groups as main,
                      liveuser_group_names as names
                    WHERE 
                      names.group_id = main.group_id AND 
                      names.language_id = '.(int)$this->_langs[$language];
                      
            $_groups = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
            if (DB::isError($_groups)) {
                return $_groups;
            };
            if (!is_array($_groups)) {
                return $_groups;
            };
            $groups = array();
            foreach($_groups as $key => $value) {
                $value['rights'] = $this->getGroupRights($value['id']);
                $groups[$value['id']] = $value;
            };
            
            $sql = 'SELECT 
                      group_id,
                      subgroup_id
                    FROM 
                      liveuser_group_subgroups';
            $subgroups = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
            if (DB::isError($subgroups)) {
                return $subgroups;
            };
            
            foreach($groups as $id => $group) {
                $in_array = false;
                foreach($subgroups as $subgroup) {
                    if ($id == $subgroup['subgroup_id']) {
                        $in_array = true;
                        break;
                    };
                };
                if (!$in_array) {
                    $result[$id] = $group;
                    $result[$id]['subgroups'] = $this->getAllGroups($language, $with_rights, $id);
                };
            };
        } else {
            foreach($subgroups as $subgroup) {
                if ($subgroup['group_id'] == $parentgroup) {
                    $result[$subgroup['subgroup_id']] = $groups[$subgroup['subgroup_id']];
                    $result[$subgroup['subgroup_id']]['subgroups'] = $this->getAllGroups($language, $with_rights, $subgroup['subgroup_id']);
                };
            };
        };
        return $result;
    }
    
    
    /**
     * Get list of all applications
     *
     * Only applications are listed, which have a name set in the
     * specified language. if the optional second param is
     * set to TRUE, areas inside the app are listet too.
     * Arraystructure is like this:
     * array(
     *   application_id => array(
     *      'name' => 'xyz',
     *      'areas' => array(
     *          area_id => array(
     *              'name' => 'xyz',
     *              ...
     *          ), ...
     *      ),
     *      ...
     *   ), ...
     * );
     *
     * @param string  $language    language (two letter code) of name/comment of areas/rights
     * @param boolean $with_areass (optional) list areas inside the apps too (calls getAllAreas() method)
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getAllApplications($language, $with_areas = false)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.application_id as id,
                  names.application_name as name,
                  names.application_comment as comment,
                  main.application_define_name as define_name
                FROM
                  liveuser_applications as main,
                  liveuser_application_names as names
                WHERE
                  main.application_id = names.application_id AND
                  names.language_id = '.(int)$this->_langs[$language].'
                ORDER BY 
                  main.application_id ASC';
        $applications = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($applications)) {
            return $applications;
        }
        
        if ($with_areas == true) {
	        $_applications = array();
	        foreach($applications as $key => $value) {
	            $id = $value['id'];
	            unset($value['id']);
	            $areas = $this->getAllAreas($language, $id);
		        if (is_array($areas)) {
		        	$_applications[$id] = $value;
		            foreach($areas as $area_id => $area) {
		                if ($area['application_id'] == $id) {
		                    unset($area['application_id']);
		                    $_applications[$id]['areas'][$area_id] = $area;
		                    unset($areas[$area_id]);
		                }
	            	}    
		        }
	        }
            return $_applications;
        }
        return $applications;
    }
    
    /**
     * Get the all data for one application including name and comment in the defined language.
     *
     * @param string  $language    language (two letter code) of name/comment of application
     * @param boolean $application_id	Primary key of the application to retrieve
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getApplication($language, $application_id)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.application_id as id,
                  names.application_name as name,
                  names.application_comment as comment,
                  main.application_define_name as define_name
                FROM
                  liveuser_applications as main,
                  liveuser_application_names as names
                WHERE
        		  main.application_id='.$application_id.' AND
                  main.application_id = names.application_id AND
                  names.language_id = '.(int)$this->_langs[$language].'
                ORDER BY 
                  main.application_id ASC';
        $application = $this->dbc->getRow($sql, DB_FETCHMODE_ASSOC);
        if (DB::isError($application)) {
            return $application;
        }
        
        return $application;
    }
    
    
    
    
    /**
     * Get list of all areas within a given application
     *
     * Only areas are lists, which have a name set in the
     * specified language. if the optional second param is
     * set to TRUE, rights inside the areas are listet too.
     * Arraystructure is like this:
     * array(
     *   area_id => array(
     *      'name' => 'xyz',
     *      'rights' => array(
     *          right_id => array(
     *              'name' => 'xyz',
     *              ...
     *          ), ...
     *      ),
     *      ...
     *   ), ...
     * );
     *
     * @param string  $language    language (two letter code) of name/comment of areas/rights
     * @param integer $application_id	Primary key of the application of which the areas shall be retrieved
     * @param boolean $with_rights (optional) list rights inside the areas too (calls getAllRights() method)
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getAllAreas($language, $application_id, $with_rights = false)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.area_id as id,
        		  main.application_id as application_id,
                  names.area_name as name,
                  names.area_comment as comment,
                  main.area_define_name as define_name
                FROM
                  liveuser_areas as main,
                  liveuser_area_names as names
                WHERE
        		  main.application_id='.$application_id.' AND 
                  main.area_id = names.area_id AND
                  names.language_id = '.(int)$this->_langs[$language].'
                ORDER BY 
                  main.area_id ASC';
        $areas = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($areas)) {
            return $areas;
        }
        
        $rights = $this->getAllRights($language);
        if (DB::isError($rights)) {
            return $rights;
        };
        
        $_areas = array();
        foreach($areas as $key => $value)
        {
            $id = $value['id'];
            unset($value['id']);
            $_areas[$id] = $value;
            foreach($rights as $right_id => $right) {
                if ($right['area_id'] == $id) {
                    unset($right['area_id']);
                    $_areas[$id]['rights'][$right_id] = $right;
                    unset($rights[$right_id]);
                };
            };
        }
        
        return $_areas;
    }
    
    
    /**
     * Get data for one area
     *
     * Returns the area name in the given language, together with 
     * the corresponding area comment and the global define name as
     * an associative array.
     *
     * @param string  $language    language (two letter code) of name/comment of areas/rights
     * @param int $area_id         Database ID of the area
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getArea($language,$area_id)    
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.area_id as id,
                  main.application_id as application_id,
                  names.area_name as name,
                  names.area_comment as comment,
                  main.area_define_name as define_name
                FROM
                  liveuser_areas as main,
                  liveuser_area_names as names
                WHERE
                  main.area_id = '.(int)$area_id.' AND
                  main.area_id = names.area_id AND
                  names.language_id = '.(int)$this->_langs[$language].'
                ORDER BY 
                  main.area_id ASC';
        $areas = $this->dbc->getRow($sql, DB_FETCHMODE_ASSOC);
       
        return $areas;
    }
    
    
    /**
     * Get list of all languages
     *
     * If the optional first parameter is set to TRUE,
     * translations of languagenames are listet too.
     * Arraystructure is like this:
     * array(
     *   two_letter_code => array(
     *      'id' => 123
     *      'name' => 'xyz',
     *      'translations' => array(
     *          two_letter_code => translation of name,
     *          ...
     *          ), ...
     *      ),
     *      ...
     *   ), ...
     * );
     *
     * @param boolean $translations (optional) list translations too
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getAllLanguages($translations = false)
    {
        $sql = 'SELECT
                  language_id as id,
                  two_letter_name as two_letter_code,
                  native_name as name
                FROM 
                  liveuser_languages';
        $langs = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($langs)) {
            return $langs;
        };
        
        if ($translations) {
            $sql = 'SELECT
                      names.native_language_id as native_language_id,
                      main.two_letter_name as two_letter_code,
                      names.native_name as name
                    FROM
                      liveuser_languages as main,
                      liveuser_language_names as names
                    WHERE
                      main.language_id = names.language_id';
            $trans = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
            if (DB::isError($trans)) {
                return $trans;
            };
        };
        
        foreach($langs as $key => $value) {
            unset($langs[$key]);
            $code = $value['two_letter_code'];
            unset($value['two_letter_code']);
            $langs[$code] = $value;
            
            if ($translations) {
                foreach($trans as $translation) {
                    if ($translation['native_language_id'] == $value['id']) {
                        $langs[$code]['translations'][$translation['two_letter_code']] = $translation['name'];
                    };
                };
            };
        }
        return $langs;
    }
    
    /**
     * Get list of all users
     *
     * Arraystructure is like this:
     * array(
     *   user_id => array(
     *      'handle' => 'xyz'
     *      ...
     *   ), ...
     * );
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getAllUsers()
    {
        $sql = 'SELECT 
                  user_id,
                  handle,
                  password,
                  type,
                  currentlogin,
                  lastlogin,
                  owner_user_id,
                  owner_group_id,
                  is_active
                FROM 
                  liveuser_users';
        $users = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($users)) {
            return $users;
        };
        
        $_users = array();
        foreach($users as $key => $value)
        {
            $user_id = $value['user_id'];
            unset($value['user_id']);
            $_users[$user_id] = $value;
        }
        
        return $_users;
    }
    
    /**
     * Get one users
     *
     * Arraystructure is like this:
     * array(
     *   'handle' => 'xyz'
     *   ...
     * );
     *
     * @param string $user_id user id
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getUser($user_id)
    {
        $sql = 'SELECT 
                  handle,
                  password,
                  type,
                  currentlogin,
                  lastlogin,
                  owner_user_id,
                  owner_group_id,
                  is_active
                FROM 
                  liveuser_users
                WHERE
                  user_id = '.$this->dbc->quote($user_id);
        $user = $this->dbc->getRow($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($user)) {
            return $user;
        };
        
        // Add user rights
        $user['rights'] = $this->getUserRights($user_id);
        if (DB::isError($user['rights'])) {
            return $user['rights'];
        };
        
        // Add groups, user is a member in
        $user['groups'] = $this->getUserGroups($user_id);
        if (DB::isError($user['groups'])) {
            return $user['groups'];
        };
        
        return $user;
    }
    
    /**
     * Get list of all rights
     *
     * Only rights are lists, which have a name set in the
     * specified language. Arraystructure is like this:
     * array(
     *   right_id => array(
     *      'area_id' => 123
     *      'name' => 'xyz',
     *      ...
     *   ), ...
     * );
     *
     * @param string $language    language (two letter code) of name/comment of rights
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getAllRights($language)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.right_id as id,
                  main.area_id as area_id,
                  main.right_define_name as define_name,
                  main.has_implied as has_implied,
                  main.has_level as has_level,
                  main.has_scope as has_scope,
                  names.right_name as name,
                  names.right_comment as comment
                FROM
                  liveuser_rights as main,
                  liveuser_right_names as names
                WHERE
                  main.right_id = names.right_id AND
                  names.language_id = '.(int)($this->_langs[$language]).'
                ORDER BY 
                  main.area_id ASC';
        $rights = $this->dbc->getAll($sql, NULL, DB_FETCHMODE_ASSOC);
        if (DB::isError($rights)) {
            return $rights;
        };
        
        $_rights = array();
        foreach($rights as $key => $value)
        {
            $id = $value['id'];
            unset($value['id']);
            $_rights[$id] = $value;
        }
        
        return $_rights;
    }
    
    /**
     * Get data for one specific right
     *
     * Returns an array of this structure:
     * array('id' =>..., 'area_id' =>..., 'define_name' =>..., 'has_implied' =>...,
     *       'has_level' =>..., 'has_scope' =>..., 'name' =>..., 'comment' =>...);
     *
     * @param string $language    language (two letter code) of name/comment of rights
     *
     * @access public
     * @return mixed array or DB Error object
     **/
    function getRight($language, $right_id)
    {
        // Get all language ids
        if (DB::isError($result = $this->_getLanguages())) {
            return $result;
        };
            
        // Check if language is a known one
        if (!isset($this->_langs[$language])) {
            return false;
        };
        
        $sql = 'SELECT 
                  main.right_id as id,
                  main.area_id as area_id,
                  main.right_define_name as define_name,
                  main.has_implied as has_implied,
                  main.has_level as has_level,
                  main.has_scope as has_scope,
                  names.right_name as name,
                  names.right_comment as comment
                FROM
                  liveuser_rights as main,
                  liveuser_right_names as names
                WHERE
        		  main.right_id = '.$right_id.' AND
                  main.right_id = names.right_id AND
                  names.language_id = '.(int)($this->_langs[$language]).'
                ORDER BY 
                  main.area_id ASC';
        $right = $this->dbc->getRow($sql, DB_FETCHMODE_ASSOC);
        if (DB::isError($right)) {
            return $right;
        };
        
        return $right;
    }
    
    
    
    /**
     * LiveUser_Perm_Container_DB_Complex_Admin::checkArea()
     * 
     * 
     * Note: This funktion doesn't work currently.
     *
     * Checks if the current user has a right in a
     * given area. If no second parameter is given, this also
     * checks if the user is logged in and will only return
     * true if he has the right in question AND is currently
     * logged in. Set the second parameter to false if you
     * just want to check wheter the user has a certain
     * right or not while ignoring the login status.
     *
     * @access  public
     * @param   string   $area_id  Id of the AuthArea to use
     * @param   boolean  $checkLogin  whether we should also check is the user is logged in as well
     * @return  boolean  See function description.
     **/
    function checkArea($area_id, $checkLogin = true)
    {
        $hasright = false;

        if ($this->rights) {
            if ($this->user_type > LIVEUSER_AREAADMIN_TYPE_ID
               || ($this->user_type == LIVEUSER_AREAADMIN_TYPE_ID
               && $this->area_admin && in_array($area_id, $this->area_admin)))
            {
                $hasright = true;
            // Check if user has rights in this area at all.
            } elseif (is_array($this->rights[$area_id])) {
                // Now let's see if we shall check the login status, too
                if ($checkLogin === true) {
                    // Yep, login has to be checked, so we'll only return true
                    // if the user is currently logged in
                    if ($this->logged_in == true) {
                        $hasright = true;
                    }
                } else {
                    // We know the user has the right and we don't want to check
                    // the login status, so true will be returned.
                    $hasright = true;
                }
            }
        } elseif ($this->ondemand) {
            $this->readRights();
            checkArea($area_name, $checkLogin);
        }
        return $hasright;
    } // end func checkArea
}

?>
