<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * DB_Simple_Admin container for maintaining Auth/DB and Perm/DB_Simple
 *
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::DB class.
 */
require_once 'LiveUser/LoginManager.php';
require_once 'DB.php';

/**
 * Class LiveUser_Admin_Container_DB_Simple
 *
 * Simple DB-based complexity driver for LiveUser.
 *
 * Description:
 * This admin class provides the following functionalities
 * - adding users
 * - removing users
 * - changing user data (auth related: username, pwd, active)
 * - adding rights
 * - removing rights
 * - get all users
 *
 * ATTENTION:
 * This class is only experimental. API may change. Use it at your own risk.
 *
 * @author  Bjrn Kraus <krausbn@php.net>
 * @version $Id: DB_Simple_Admin.php,v 1.2 2003/05/22 23:37:47 krausbn Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Admin_Container_DB_Simple
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused).
     *
     * @var    string
     * @access private
     */
    var $dsn = NULL;

    /**
     * PEAR::DB connection object.
     *
     * @var    object
     * @access private
     */
    var $dbc = NULL;

    /**
     * Table prefix
     * Prefix for all db tables the container has.
     *
     * @var    string
     * @access public
     */
    var $prefix = 'liveuser_';

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var    boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * Constructor
     *
     * @param  mixed    $connectOptions   Array or PEAR::DB object.
     * @return void
     */
    function &LiveUser_Admin_Container_DB_Simple(&$connectOptions)
    {
        if (DB::isConnection($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            $this->dbc = DB::connect($connectOptions['dsn']);

            if (!DB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }
    } // end func LiveUser_Admin_Container_DB_Simple

    /**
     * Adds a new user to Auth/DB and Perm/DB_Simple.
     *
     * @access  public
     * @param   string     $handle    Handle (username).
     * @param   string     $password  Password (optional).
     * @param   boolean    $active    Sets the user active (1) or not (0) (optional).
     * @param   int        $type      User type (constants defined in Perm/Common.php) (optional).
     * @return  mixed                 Users auth ID on success, DB error if not.
     */
    function addUser($handle, $password = '', $active = 1, $type = 1)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Generate new user ID
        $authId = md5($handle.getmypid().uniqid(rand(),1));

        // Register new user in auth table
        $query = '
            INSERT INTO
                '.$this->prefix.'users
                (user_id, handle, password, is_active)
            VALUES
                (
                '.$this->dbc->quote($authId).',
                '.$this->dbc->quote($handle).',
                '.$this->dbc->quote($password).',
                '.$this->dbc->quote(($active ? 'Y' : 'N')).'
                )';
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        // Register new user in auth table
        $query = '
            INSERT INTO
                '.$this->prefix.'perm_users
                (user_id, auth_user_id, type, auth_container_name)
            VALUES
                (
                '.$this->_getNewPermId().',
                '.$this->dbc->quote($authId).',
                '.(int)$type.',
                '.$this->dbc->quote('DB').'
                )';
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        return $authId;
    } // end func addUser

    /**
     * Removes an existing user from Auth/DB and Perm/DB_Simple.
     *
     * @access  public
     * @param   string   $authId   Auth user ID of the user that should be removed.
     * @return  mixed              True on success, DB error if not.
     */
    function removeUser($authId)
    {
        if (!$this->init_ok) {
            return false;
        }

        $permId = $this->_getPermUserId($authId);

        // Delete user from auth table (DB/Auth)
        $query = '
            DELETE FROM
                '.$this->prefix.'users
            WHERE
                user_id = '.$this->dbc->quote($authId);
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        // Delete user's rights (Perm/DB)
        $query = '
            DELETE FROM
                '.$this->prefix.'userrights
            WHERE
                user_id = '.(int)$permId;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        // Delete user from perm table (Perm/DB)
        $query = '
            DELETE FROM
                '.$this->prefix.'perm_users
            WHERE
                user_id = '.(int)$permId;
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        return true;
    } // end func removeUser

    /**
     * Changes user data in auth table.
     *
     * @access  public
     * @access  string     $authId    Auth user ID.
     * @param   string     $handle    Handle (username) (optional).
     * @param   string     $password  Password (optional).
     * @param   boolean    $active    Sets the user active (1) or not (0) (optional).
     * @return  mixed                 True on success, DB error if not.
     */
    function changeUser($authId, $handle = '', $password = '', $active = NULL)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Create query.
        $query = '
            UPDATE
                '.$this->prefix.'users
            SET ';

        if (!empty($handle)) {
            $updateValues[] = 'handle = '.$this->dbc->quote($handle);
        }
        if (!empty($password)) {
            $updateValues[] = 'password = '.$this->dbc->quote($password);
        }
        if ($active) {
            $updateValues[] = 'is_active = '.$this->dbc->quote($active ? 'Y' : 'N');
        }

        if (count($updateValues) >= 1) {
            $query .= implode(', ', $updateValues);
        } else {
            return false;
        }

        $query .= '
            WHERE
                user_id = '.$this->dbc->quote($authId);
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        return true;
    } // end func changeUser

    /**
     * Adds rights for a single user.
     *
     * @access  public
     * @access  string  $authId  Auth user ID.
     * @param   array   $rights  Array of right IDs to add.
     * @return  mixed            True on success, DB error if not.
     */
    function addRights($authId, $rights)
    {
        if (!$this->init_ok) {
            return false;
        }

        // Insert rights.
        foreach ($rights as $right) {
            $query = '
                INSERT INTO
                    '.$this->prefix.'userrights
                    (user_id, right_id)
                VALUES
                    (
                    '.$this->dbc->quote($this->_getPermUserId($authId)).',
                    '.(int)$right.'
                    )';
            $result = $this->dbc->query($query);
            if (DB::isError($result)) {
                return $result;
            }
        }

        return true;
    } // end func addRights

    /**
     * Removes rights for a single user.
     *
     * @access  public
     * @access  string  $authId  Auth user ID.
     * @param   array   $rights  Array of right IDs to remove or empty to
     *                           remove all.
     * @return  mixed            True on success, DB error if not.
     */
    function removeRights($authId, $rights = NULL)
    {
        if (!$this->init_ok) {
            return false;
        }

        $query = '
            DELETE FROM
                '.$this->prefix.'userrights
            WHERE
                user_id = '.$this->_getPermUserId($authId);

        if (is_array($rights)) {
            $query .= ' AND right_id IN ('.implode(', ', $rights);
        }

        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        return true;
    } // end func removeRights

    /**
     * Shortcut to delete all existing rights of a user and add the given.
     *
     * @access  public
     * @access  string  $authId  Auth user ID.
     * @param   array   $rights  Array of right IDs to add.
     * @return  mixed            True or DB error if not.
     */
    function changeRights($authId, $rights)
    {
        $this->removeRights($authId);
        $this->addRights($authId, $rights);
        return true;
    } // end func changeRights

    /**
     * Gets all users with handle, password, authId, permId, currentLogin,
     * lastLogin, isActive and individual rights.
     *
     * The array will look like this:
     * $userData[0]['authId']       = '1';
     *             ['permId']       = 'wujha433gawefawfwfiuj2ou9823r98h';
     *             ['handle']       = 'myLogin';
     *             ['password']     = 'd346gs2gwaeiuhaeiuuweijfjuwaefhj';
     *             ['currentLogin'] = 1254801292; (Unix timestamp)
     *             ['lastLogin']    = 1254801292; (Unix timestamp)
     *             ['isActive']     = 1; (1 = yes, 0 = no)
     *             ['type']         = 1;
     *             ['container']    = 'DB';
     *             ['rights'][0]    = 1;
     *             ['rights'][1]    = 4;
     *             ['rights'][2]    = 5;
     *
     * @access  public
     * @param   boolean  $withRights  If true the rights for each user will be retrieved.
     * @return  mixed                 Array with user data or DB error.
     */
    function getAllUsers($withRights = false)
    {
        if (!$this->init_ok) {
            return false;
        }

        // First: Get all data from auth and perm table.
        $query = '
            SELECT
                U.user_id AS authId,
                U.handle,
                U.password,
                U.currentlogin AS currentLogin,
                U.lastlogin AS lastLogin,
                U.is_active AS isActive,
                PU.user_id AS permId,
                PU.type,
                PU.auth_container_name AS container
            FROM
                '.$this->prefix.'users AS U
            INNER JOIN
                '.$this->prefix.'perm_users AS PU
            ON
                U.user_id = PU.auth_user_id
            ORDER BY
                U.handle ASC';
        $userData = $this->dbc->getAssoc($query, true, array(), DB_FETCHMODE_ASSOC);
        if (DB::isError($userData)) {
            return $userData;
        }

        // Now: Check if we should get the rights for each user.
        if ($withRights) {
            $query = '
                SELECT
                    *
                FROM
                    '.$this->prefix.'userrights
                ORDER BY
                    user_id ASC';
            $result = $this->dbc->query($query);
            if (DB::isError($result)) {
                return $result;
            }

            $rights = array();

            while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
                if (isset($rights[$row['user_id']])) {
                    array_push($rights[$row['user_id']], $row['right_id']);
                } else {
                    $rights[$row['user_id']][0] = $row['right_id'];
                }
            }


            $udCount = count($userData);
            $keys = array_keys($userData);

            for ($i = 0; $i < $udCount; $i++) {
                if (is_array($rights[$userData[$keys[$i]]['permId']])) {
                    $userData[$keys[$i]]['rights'] = $rights[$userData[$keys[$i]]['permId']];
                    unset($rights[$userData[$keys[$i]]['permId']]);
                }
            }
        }

        return $userData;
    } // end func getAllUsers

    /**
     * Updates user type.
     *
     * @access  public
     * @param   string  $authId  Auth user ID.
     * @param   int     $type    User type (constants defined in Perm/Common.php).
     * @return  mixed            True on success, DB error if not.
     */
    function setUserType($authId, $type)
    {
        if (!$this->init_ok) {
            return false;
        }

        $query = '
            UPDATE
                '.$this->prefix.'perm_users
            SET
                type = '.(int)$type.'
            WHERE
                auth_user_id = '.$this->dbc->quote($authId);
        $result = $this->dbc->query($query);
        if (DB::isError($result)) {
            return $result;
        }

        return true;
    } // end func setUserType

    /**
     * Gets the perm ID of a user.
     *
     * @access  privat
     * @access  string  $authId  Auth user ID.
     * @return  mixed            Permission ID or DB error.
     */
    function _getPermUserId($authId)
    {
        // Store persistent information we allready have.
        static $permIds = array();

        if (!isset($permIds[$authId])) {
            $query = '
                SELECT
                    auth_user_id
                FROM
                    '.$this->prefix.'perm_users
                WHERE
                    user_id = '.$this->dbc->quote($authId);
            $permIds[$authId] = $this->dbc->getOne($query);
            if (DB::isError($result)) {
                return $result;
            }
        // Remind myself: What happens if authId isn't in the table?
        //                Do we get a db error our is $permId empty?
        }

        return $permIds[$auth];
    } // end func _getPermUserId

    /**
     * Gets the perm ID of a user.
     *
     * @access  privat
     * @access  string  $authId  Auth user ID.
     * @return  mixed            Permission ID or DB error.
     */
    function _getPermUserId($authId)
    {
        // Store persistent information we allready have.
        static $permIds = array();

        if (!isset($permIds[$authId])) {
            $query = '
                SELECT
                    user_id
                FROM
                    '.$this->prefix.'perm_users
                WHERE
                    auth_user_id = '.$this->dbc->quote($authId);
            $permIds[$authId] = $this->dbc->getOne($query);
        }

        return $permIds[$authId];
    } // end func _getPermUserId

    /**
     * Gets a new perm ID.
     *
     * @access  privat
     * @return  mixed            Permission ID or DB error.
     */
    function _getNewPermId()
    {
        $query = '
            SELECT
                MAX(user_id)
            FROM
                '.$this->prefix.'perm_users';
        $newPermId = $this->dbc->getOne($query) + 1;
        if (DB::isError($result)) {
            return $result;
        }

        return $newPermId;
    } // end func _getNewPermId
}
?>