<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * LiveUser is a framework designed to be flexible.
 *
 * Since it is impossible to have a
 * "one size fits all" it takes a container
 * approach which should enable it to
 * be versatile enough to meet most needs.
 *
 * @package  LiveUser
 * @category authentication
 */

/**#@+
 * Error related constants definition
 *
 * @var integer
 */
define('LIVEUSER_ERROR',                        -1);
define('LIVEUSER_NOT_SUPPORTED',                -2);
define('LIVEUSER_CONFIG_NOT_FOUND',             -3);
define('LIVEUSER_CONFIG_UNKNOWN_TYPE',          -4);
define('LIVEUSER_CONFIG_PHP_MISSING_ARRAY',     -5);
define('LIVEUSER_MISSING_DEPS',                 -6);
define('LIVEUSER_MISSING_LOGINFUNCTION',        -7);
define('LIVEUSER_CONFIG_ARRAY_NOT_AN_ARRAY',    -8);
define('LIVEUSER_MISSING_LOGOUTFUNCTION',       -9);
define('LIVEUSER_CANNOT_STORE_COOKIE_ID',      -10);
define('LIVEUSER_CANNOT_ACCESS_COOKIE_STORE',  -11);
define('LIVEUSER_CANNOT_RETRIEVE_AUTH_OBJECT', -12);
/**#@-*/

/**#@+
 * Status of the current user.
 *
 * @var integer
 */
define('LIVEUSER_AUTH_IDLED',       -1);
define('LIVEUSER_AUTH_EXPIRED',     -2);
define('LIVEUSER_AUTH_WRONG_LOGIN', -3);
define('LIVEUSER_AUTH_ISINACTIVE',  -4);
/**#@-*/

/**
 * Class LiveUser - Login handling class
 *
 * Description:
 * This is a manager class for a user login system using the LiveUser
 * class. It creates a LiveUser object, takes care of the whole login
 * process and stores the LiveUser object in a session.
 * The whole process is completely customizable via an XML configuration
 * file.
 * You can also configure this class to try to connect to more than
 * one server that can store user information - each server requiring
 * a different backend class. This way you can for example create a login
 * system for a live website that first queries the local database and
 * if the requested user is not found, it tries to find im in your
 * company's LDAP server. That way you don't have to create lots of
 * user accounts for your employees so that they can access closed
 * sections of your website - everyone can use his existing account.
 *
 * NOTE: No browser output may be made before using this class, because
 * it will try to send HTTP headers such as cookies and redirects.
 *
 * Requirements:
 * - Package PEAR::Cache_Lite
 * - Package PEAR::XML_Tree
 * - Should run on PHP version 4.1.0 or higher, tested only from 4.2.1 onwards
 *
 * Thanks to:
 * Bjoern Schotte, Kristian Koehntopp, Antonio Guerra
 *
 * @author   Markus Wolff       <wolff@21st.de>
 * @author   Bjoern Kraus       <krausbn@php.net>
 * @author   Lukas Smith        <smith@backendmedia.com>
 * @author   Pierre-Alain Joye  <pajoye@php.net>
 * @author   Arnaud Limbourg    <arnaud@php.net>
 * @version  $Id: LiveUser.php,v 1.10 2003/05/25 13:11:31 arnaud Exp $
 * @package  LiveUser
 * @category authentication
 */
class LiveUser
{
    /**
     * The auth container object.
     *
     * @access private
     * @var    object
     */
    var $_auth = null;

    /**
     * The permission container object
     *
     * @access private
     * @var    object
     */
    var $_perm = null;

    /**
     * The session name to be used for all sessions
     *
     * @var string
     */
    var $sessionName = 'LiveUser';

    /**
     * The name of the session variable in which the auth object
     * will be stored
     *
     * @var string
     */
    var $sessionVarName = 'ludata';

    /**
     * The variable name for the user handle that comes from a
     * HTML login form
     *
     * @var string
     */
    var $handleParamName = 'handle';

    /**
     * The variable name for the password that comes from a
     * HTML login form
     *
     * @var string
     */
    var $passwdParamName = 'passwd';

    /**
     * The input channel in which to look for login information.
     * Can be POST for form input or GET for URL input.
     *
     * @var string [GET|POST]
     */
    var $loginInputType = 'POST';

    /**
     * Name of the GET-Parameter that is being monitored for logging
     * out. If present, the auth object in the session will be
     * destroyed.
     *
     * @var string
     */
    var $logoutParamName = 'logout';

    /**
     * Function to be called when the user wants to logout.
     *
     * @access public
     * @var    string
     */
    var $logoutFunction = '';


    /**
     * Function to be called when the user is not logged in.
     *
     * @access public
     * @var    string
     */
    var $loginFunction = '';

    /**
     * Flag to run the init() method when the constructor is called.
     *
     * @access public
     * @var    boolean
     */
    var $autoInit = false;

    /**
     * Flag: Should the user be forced to login?
     * If set to true the login screen will be displayed until the login
     * is successfull.
     *
     * @access public
     * @var    boolean
     */
    var $forceLogin = false;

    /**
     * The name for the cookie to store auth information in (used
     * for the "remember me" function).
     *
     * @var string
     */
    var $cookieName = 'loginData';

    /**
     * This flag determines if "remember me" cookies are allowed. If
     * not, the LoginManager won't even look for a cookie, so login
     * information has to be re-entered for every session.
     * This is the recommended behaviour!!!
     * Set to 0 for not allowing cookies, set to 1 for allowing them.
     *
     * @var integer [0|1]
     */
    var $allowCookies = false;

    /**
     * The name of the "remember me" checkbox on a HTML form.
     * If this is provided via the channel defined in loginInputType (GET or POST),
     * and allowCookies has been set to 1, a cookie with user information
     * will be set so the user won't have to login using his username and
     * password again as long as the cookie exists.
     *
     * @var string
     */
    var $cookieParamName = 'useCookie';

    /**
     * The lifetime of the "remember me" cookie in days.
     * Note that the cookie lifetime will be refreshed
     * every time the user logs in.
     * @var integer
     */
    var $cookieLifetime = 30;

    /**
     * The domain for the "remember me" cookie. Example: "21st.de"
     *
     * @var string
     */
    var $cookieDomain = 'localhost';

    /**
     * The path for the "remember me" cookie. Example: "/"
     *
     * @var string
     */
    var $cookiePath = '/';

    /**
     * The URL to redirect to when the user logs out
     *
     * @var string
     */
    var $logoutRedirectURL = '';

    /**
     * Determines whether or not to destroy the entire session on logout
     *
     * @var boolean
     */
    var $destroySessionOnLogout = true;

    /**
     * Nested array with the auth containers that shall be queried for user information.
     * Format:
     * <code>
     * array(1 => array("option1" => "value", ....))
     * </code>
     * Typical options are:
     * <ul>
     * - server: The adress of the server being queried (ie. "localhost").
     * - handle: The user name used to login for the server.
     * - password: The password used to login for the server.
     * - database: Name of the database containing user information (this is
     *   usually used only by RDBMS).
     * - baseDN: Obviously, this is what you need when using an LDAP server.
     * - connection: Present only if an existing connection shall be used. This
     *   contains a reference to an already existing connection resource or object.
     * - type: The container type. This option must always be present, otherwise
     *   the LoginManager can't include the correct container class definition.
     *</ul>
     * @var array
     */
    var $authContainers = array();

    /**
     * Name of the session variable that is used to store information on the
     * backend in which the auth data resides.
     * This is needed to include the
     * correct backend class definition on demand.
     *
     * @var string
     */
    var $authContainerTypeVar = 'authContainerType';

    /**
     * Array of settings for the permission container to use for retrieving
     * user rights.
     * If set to false, no permission container will be used.
     * If that is the case, all calls to checkRight() will return false.
     * The array element 'type' must be present for the LoginManager to be able
     * to include the correct class definition (example: "DB_Complex").
     *
     * @var mixed
     */
    var $permContainer = false;

    /**
     * Has the auth session expired?
     *
     * @var boolean
     * @see init(), processLogout()
     */
    var $expired = false;

    /**
     * Is the maximum idletime over?
     *
     * @var boolean
     * @see init(), processLogout()
     */
    var $idled = false;

    /**
     * Current authentication status
     *
     * @var string
     */
    var $status = null;

    /**
     * Posible Types of the config file and it's import function.
     *
     * @var array
     */
    var $configType = array('PHP' => '_readConfigFilePHP',
                            'XML' => '_readConfigFileXML');

    /**
     * Whether an error raised
     *
     * @access private
     * @var    boolean
     */
    var $_error = false;

    /**
     * Array containing errors
     *
     * @var array
     */
    var $_errors = array();

    /**
     * Key used for encrytion purposes
     *
     * @var string
     */
    var $_cookieEncryptionKey = 'LIVEUSERPERMMANAGEMENT';

    /**
     * Checks the given file and returns an object of the LoginManager.
     *
     * @access public
     * @param  string   $configFile The config file or config array to configure several.
     * @param  mixed    $options    Options for the file import.
     * @param  string   $type       File type of the given file.
     * @return object   Returns an object.
     */
    function &create($configFile = '', $options = false, $type = '')
    {
        if ((!is_file($configFile)) && (!is_array($configFile))) {
            return LiveUser::raiseError(
                LiveUser::errorMessage(LIVEUSER_CONFIG_NOT_FOUND),
                LIVEUSER_CONFIG_NOT_FOUND
            );
        }
        $obj = &new LiveUser($configFile, $options, $type);
        return $obj;
    }

    /**
     * LiveUser::LiveUser()
     *
     * Class constructor. Optionally, a valid filepath to an XML or PHP configuration file
     * or a configuration array can be passed.
     *
     * @param  string   $configFile The config file or the config array to configure.
     * @param  mixed    $options    Options for the file import.
     * @param  string   $type       File type of the given file.
     * @access public
     */
    function LiveUser($configFile = '', $options = false, $type = '')
    {
        if (is_array($configFile)) {
            $this->readConfigArray($configFile);
        } else if ($configFile != '') {
            $this->readConfigFile($configFile, $options, $type);
        }
        if ($this->autoInit) {
            $this->init();
        }
    }

    /**
     * Simple method instantiating a PEAR_Error object
     * when an error occurs.
     *
     * It is a copy/paste/edit from PEAR::raiseError.
     *
     * @param mixed  $message  a text error message or a PEAR error object
     *
     * @param int    $code     a numeric error code (it is up to your class
     *                  to define these if you want to use codes)
     *
     * @param string $userinfo If you need to pass along for example debug
     *                  information, this parameter is meant for that.
     *
     * @access public
     * @return object   a PEAR error object
     */
    function raiseError($message = null, $code = null, $userinfo = null)
    {
        include_once 'PEAR.php';

        return PEAR::raiseError($message, $code, null, null, $userinfo);
    }

    /**
     * Return a textual error message for a LiveUser error code
     *
     * @access private
     * @param  integer  $value error code
     * @return string   Error message, or false if the error code was not recognized.
     */
    function errorMessage($value)
    {
        static $errorMessages;
        if (!isset($errorMessages)) {
            $errorMessages = array(
                LIVEUSER_ERROR                       => 'Unknown error',
                LIVEUSER_NOT_SUPPORTED               => 'Feature not supported',
                LIVEUSER_CONFIG_NOT_FOUND            => 'Config file not found',
                LIVEUSER_CONFIG_UNKNOWN_TYPE         => 'Invalid configuration type',
                LIVEUSER_MISSING_DEPS                => 'Missing package depedencies',
                LIVEUSER_CONFIG_PHP_MISSING_ARRAY    => 'The config array has not been found',
                LIVEUSER_MISSING_LOGINFUNCTION       => 'Login function not found',
                LIVEUSER_CONFIG_ARRAY_NOT_AN_ARRAY   => 'The config array is not an array',
                LIVEUSER_CANNOT_STORE_COOKIE_ID      => 'Cannot store the cookie id on the server',
                LIVEUSER_CANNOT_ACCESS_COOKIE_STORE  => 'Cannot access cookie store in read/write mode',
                LIVEUSER_CANNOT_RETRIEVE_AUTH_OBJECT => 'Cannot retrieve Auth object from session'
            );
        }

        if (PEAR::isError($value)) {
            $value = $value->getCode();
        }

        return isset($errorMessages[$value]) ? $errorMessages[$value] : $errorMessages[LIVEUSER_ERROR];
    }

    /**
     * LiveUser::init()
     *
     * Tries to retrieve auth object from session.
     * If this fails, the class attempts a login based on cookie or form
     * information (depends on class settings).
     * Returns true if a auth object was successfully retrieved or created.
     * Otherwise, false is returned.
     *
     * @return boolean
     * @access public
     */
    function init()
    {
        if ($this->_error) {
            return false;
        }

        // Set the name of the current session
        session_name($this->sessionName);
        // If there's no session yet, start it now
        session_start();

        // Determine if user wanted to log out
        $this->processLogout();

        // Include needed permission container class
        if (is_array($this->permContainer)) {
            include_once 'LiveUser/Perm/Container/' . $this->permContainer['type'] . '.php';
        }

        // Try to fetch auth object from session
        if (!$this->retrieveAuthObject()) {
            if (is_null($this->_auth)) {
                $this->tryLogin();
            }
        }

        // Check if authentication session is expired.
        if (is_object($this->_auth) &&
            $this->_auth->loggedIn &&
            $this->_auth->expireTime > 0 &&
            $this->_auth->currentLogin > 0 &&
            ($this->_auth->currentLogin + $this->_auth->expireTime) < time()) {

            $this->expired = true;
            $this->status = LIVEUSER_AUTH_EXPIRED;
            $this->processLogout();
        }

        // Check if maximum idle time is reached.
        if (is_object($this->_auth) &&
            $this->_auth->loggedIn &&
            $this->_auth->idleTime > 0 &&
            isset($_SESSION[$this->sessionVarName]['idle']) &&
            ($_SESSION[$this->sessionVarName]['idle'] + $this->_auth->idleTime) < time()) {

            $this->idled = true;
            $this->status = LIVEUSER_AUTH_IDLED;
            $this->processLogout();
        }

        $_SESSION[$this->sessionVarName]['idle'] = time();

        // Force user login.
        if ($this->forceLogin && (!$this->isLoggedIn() || $this->status)) {
            if (!empty($this->loginFunction) && function_exists($this->loginFunction)) {
                call_user_func($this->loginFunction, $this);
            } else {
                $this->_error = true;
                return LiveUser::raiseError(
                    LiveUser::errorMessage(LIVEUSER_MISSING_LOGINFUNCTION),
                    LIVEUSER_MISSING_LOGINFUNCTION
                );
            }
        }

        // Return boolean that indicates whether a auth object has been created
        // or retrieved from session
        if (is_object($this->_auth) && $this->_auth->handle != '') {
            return true;
        } else {
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessage(LIVEUSER_CANNOT_RETRIEVE_AUTH_OBJECT),
                LIVEUSER_CANNOT_RETRIEVE_AUTH_OBJECT);
        }
    }

    /**
     * LiveUser::tryLogin()
     *
     * Tries to log the user in by trying all the Auth containers defined
     * in the configuration file until there is a success or failure.
     *
     * @access  private
     */
    function tryLogin()
    {
        if ($this->_error) {
            return false;
        }

        $handle = '';
        $passwd = '';

        if (isset($_COOKIE[$this->cookieName])) {
            $cookie_data = unserialize($_COOKIE[$this->cookieName]);

            if (count($cookie_data) != 3) {
                $this->_error = true;
                return LiveUser::raiseError(
                    LiveUser::errorMessage(LIVEUSER_CANNOT_ACCESS_COOKIE_STORE),
                    LIVEUSER_CANNOT_ACCESS_COOKIE_STORE,
                    'Wrong data in cookie store');
            }

            $server_data = $this->_readStoreCookiePasswdId($cookie_data[0]);

            if ($server_data[0] != $cookie_data[2]) {
                $this->_error = true;
                return LiveUser::raiseError(
                    LiveUser::errorMessage(LIVEUSER_CANNOT_ACCESS_COOKIE_STORE),
                    LIVEUSER_CANNOT_ACCESS_COOKIE_STORE,
                    'passwords hashes do not match');
            }

            $handle = $cookie_data[1];
            $passwd = $server_data[1];
        } else {
            $httpvar = $this->loginInputType == 'POST' ? $_POST : $_GET;

            if (( !isset($httpvar[$this->handleParamName]) &&
                !isset($httpvar[$this->passwdParamName]) ) ||
                (isset($httpvar[$this->handleParamName]) &&
                $httpvar[$this->handleParamName] == '')
            ) {
                if (!empty($this->loginFunction) && function_exists($this->loginFunction)) {
                    call_user_func($this->loginFunction, $this);
                } else {
                    return LiveUser::raiseError(
                        LiveUser::errorMessage(LIVEUSER_MISSING_LOGINFUNCTION),
                        LIVEUSER_MISSING_LOGINFUNCTION
                    );
                }
            } else {
                $handle = $httpvar[$this->handleParamName];
                $passwd = $httpvar[$this->passwdParamName];
                if (get_magic_quotes_gpc()) {
                    $handle = stripslashes($handle);
                    $passwd = stripslashes($passwd);
                }
            }
        }

        if ($handle == '') {
            array_push($this->_errors, 'Missing handle');
            return false;
        }

        $counter     = 0;
        $userFound   = false;
        $backend_cnt = count($this->authContainers);
        $backends    = &$this->authContainers;

        //loop into auth containers
        while ($userFound == false && $backend_cnt > $counter) {
            $classname = 'LiveUser_Auth_Container_'.$backends[$counter]['type'];
            include_once 'LiveUser/Auth/Container/' . $backends[$counter]['type'] . '.php';
            $auth = &new $classname($backends[$counter]);
            $auth->login($handle, $passwd, true);
            if ($auth->loggedIn) {
                $userFound = true;
                $this->_auth = $auth;
                $this->_auth->backendArrayIndex = $counter;

                // Create permission object
                if (is_array($this->permContainer)) {
                    $permClass = 'LiveUser_Perm_Container_' . $this->permContainer['type'];
                    include_once 'LiveUser/Perm/Container/' . $this->permContainer['type'] . '.php';
                    if (isset($this->permContainer['connection'])) {
                        $this->_perm = &new $permClass($this->permContainer['connection']);
                    } else {
                        $this->_perm = &new $permClass($this->permContainer);
                    }
                    $this->_perm->init($this->_auth->authUserId);
                }

                $this->freeze($counter);
            } elseif ($auth->handle != '') {
                $this->status = LIVEUSER_AUTH_ISINACTIVE;
                $userFound  = true;
                $this->_auth = $auth;
                $this->_auth->backendArrayIndex = $counter;
                $this->freeze($counter);
            } else {
                $this->status = LIVEUSER_AUTH_WRONG_LOGIN;
            }
            $counter++;
        } // End loop into backends
    }

    /**
     * LiveUser::unfreeze()
     *
     * Gets auth and perm container objects back from session and tries
     * to give them an active database/whatever connection again
     *
     * @access private
     */
    function unfreeze()
    {
        if ($this->_error) {
            return false;
        }

        $this->_auth = unserialize($_SESSION[$this->sessionVarName]['auth']);

        if (isset($_SESSION[$this->sessionVarName]['perm'])) {
            $this->_perm = unserialize($_SESSION[$this->sessionVarName]['perm']);
        }

        // Use the same backend connection as for the first login
        $i = $this->_auth->backendArrayIndex;

        //need deeper tests
        $this->_auth->unfreeze($this->authContainers[$i]);

        if (isset($this->permContainer['connection']) && is_object($this->_perm)) {
            $this->_perm->unfreeze($this->permContainer['connection']);
        } elseif (is_object($this->_perm)) {
            $this->_perm->unfreeze($this->permContainer);
        }
    }

    /**
     * LiveUser::freeze()
     *
     * Calls the container objects freeze methods which should close
     * their database connections. After that, the container objects are
     * being serialized and stored in the session.
     *
     * @param  integer $counter
     * @access private
     */
    function freeze($counter)
    {
        if ($this->_error) {
            return false;
        }
        $this->_auth->freeze();
        if (is_object($this->_perm)) {
            $this->_perm->freeze();
        }

        // Bind objects to session
        $_SESSION[$this->sessionVarName] = array();
        $_SESSION[$this->sessionVarName]['auth'] = serialize($this->_auth);
        if (is_object($this->_perm)) {
            $_SESSION[$this->sessionVarName]['perm'] = serialize($this->_perm);
        }
        $_SESSION[$this->authContainerTypeVar] = $this->authContainers[$counter]['type'];

        $this->setRememberCookie();
    }

    /**
     * LiveUser::setRememberCookie()
     *
     * If cookies are allowed, this method checks if the user wanted
     * a cookie to be set so he doesnt have to enter handle and password
     * for his next login. If true, it will set the cookie.
     *
     * @access private
     */
    function setRememberCookie()
    {
        if ($this->_error) {
            return false;
        }

        if ($this->allowCookies && isset($_POST[$this->cookieParamName])) {
            // Calculate cookie timeout in days
            $cookieTimeout = time() + (86400 * $this->cookieLifetime);

            $httpvar = $this->loginInputType == 'POST' ? $_POST : $_GET;

            if (isset($httpvar[$this->passwdParamName])) {
                $password = $httpvar[$this->passwdParamName];
                $handle   = $httpvar[$this->handleParamName];
            } else {
                return false;
            }

            $store_id = md5($handle . $password);

            if (!$passwd_id = $this->_storeCookiePasswdId($password, $store_id)) {
                $this->_error = true;
                return LiveUser::raiseError(
                    LiveUser::errorMessage(LIVEUSER_CANNOT_STORE_COOKIE_ID),
                    LIVEUSER_CANNOT_STORE_COOKIE_ID,
                    'Cannot store cookie data on the server in ' . $store_id);
            }

            $setcookie = setcookie(
                          $this->cookieName,
                          serialize(array($store_id, $this->_auth->handle, $passwd_id)),
                          $cookieTimeout,
                          $this->cookiePath, $this->cookieDomain);

        if ($setcookie) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * LiveUser::retrieveAuthObject()
     *
     * Determines the backend type of the auth object stored in the session,
     * includes the proper class definition and then imports the object from
     * the session.
     *
     * @access private
     * @return boolean
     */
    function retrieveAuthObject()
    {
        if ($this->_error) {
            return false;
        }

        $success = false;

        if (isset($_SESSION[$this->sessionVarName]['auth'])) {
            $classFile  = 'LiveUser/Auth/Container/';
            $classFile .= $_SESSION[$this->authContainerTypeVar] . '.php';
            include_once $classFile;
            if (class_exists('LiveUser_Auth_Container_'.
                             $_SESSION[$this->authContainerTypeVar])) {
                $success = true;
                $this->unfreeze();
            }
        }
        return $success;
    }

    /**
     * LiveUser::processLogout()
     *
     * If the logout parameter has been set via GET/POST or the user is
     * idled/expired, this destroys the session object.
     *
     * @access private
     * @see    init()
     */
    function processLogout()
    {
        if ($this->_error) {
            return false;
        }

        // Has the user to be loged out?
        if (isset($_GET[$this->logoutParamName]) ||
            isset($_POST[$this->logoutParamName]) ||
            $this->idled || $this->expired) {

            // Yes indeed, destroy the session object now!
            unset($_SESSION[$this->sessionVarName]);

            // If there's a cookie, kill that one too...
            if (isset($_COOKIE[$this->cookieName])) {
                $cookieKillTime = time() - 86400;
                setcookie($this->cookieName, '', $cookieKillTime,
                          $this->cookiePath, $this->cookieDomain);
            }

            // If the session should be destroyed, do so now...
            if ($this->destroySessionOnLogout == true) {
                session_destroy();
            }

            // Finally, set the LoginManager's internal auth object back to NULL
            $this->_auth = null;

            // If a callback function is set, call it
            if (!empty($this->logoutFunction) && function_exists($this->logoutFunction)) {
                call_user_func($this->logoutFunction, $this);
            } else {
                return LiveUser::raiseError(
                    LiveUser::errorMessage(LIVEUSER_MISSING_LOGOUTFUNCTION),
                    LIVEUSER_MISSING_LOGOUTFUNCTION
                );
            }


            // If there's a URL to redirect to on logout, do so
            if ($this->logoutRedirectURL != '') {
                header('Location: '.$this->logoutRedirectURL);
            }
        }
    }

    /**
     * LiveUser::readConfigFile()
     *
     * Reads an configuration file specified in $configFile with the
     * proper method depending on file type.
     *
     * @param  string   $configFile
     * @param  mixed    $options    option for config file parsing
     * @param  string   $type       config file type
     * @return boolean
     * @access public
     */
    function readConfigFile($configFile, $options = false, $type = '')
    {
        if ($type == '') {
            $type = strtoupper(substr($configFile,strlen($configFile)-3));
        }

        if (is_file($configFile)) {
            if ( isset($this->configType[$type]) ) {
                $methodName = $this->configType[$type];
                return $this->$methodName($configFile, $options);
            } else {
                return false;

            }
        } else {
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessage(LIVEUSER_CONFIG_UNKNOWN_TYPE),
                LIVEUSER_CONFIG_UNKNOWN_TYPE
            );
        }
    }

    /**
     * LiveUser::_readConfigFilePHP()
     *
     * Reads a PHP configuration file specified in $configFile with
     * $arrayName as the configuration array and sets the instance vars
     * to the values found.
     *
     * @param  string   $configFile
     * @param  string   $arrayName     name of config array
     * @return boolean
     * @access private
     */
    function _readConfigFilePHP($configFile, $arrayName = '')
    {
        @include $configFile;
        if ($arrayName == '') {
            $arrayName = 'liveuserConfig';
        }
        if (isset(${$arrayName}) && is_array(${$arrayName})) {
            foreach (${$arrayName} as $key => $value) {
                $this->$key = $value;
            }
            return true;
        }

        $this->_error = true;
        return LiveUser::raiseError(
            LiveUser::errorMessage(LIVEUSER_CONFIG_PHP_MISSING_ARRAY),
            LIVEUSER_CONFIG_PHP_MISSING_ARRAY
        );
    }

    /**
     * LiveUser::_readConfigFileXML()
     *
     * Reads an XML configuration file specified in $configFile and sets
     * the class properties accordingly. Not only does this save a lot of
     * typing, but also allows several applications that use this class to
     * have the same configuration by using one central XML file.
     *
     * @param  string   $configFile     path to the config file, relative or absolute
     * @param  boolean  $options        whether to enable caching for the xml file
     *                                  'force'=>boolean    wether to force a config read
     *                                  'cacheDir'=>string  cache folder
     *                                  'lifeTime'=>boolean cache lifetime
     *                                  'cacheName'=>boolean name of the cache data,
     *                                                       @see Cache_Lite
     * @return boolean
     * @access private
     */
    function _readConfigFileXML($configFile, $cacheOptions = false)
    {
        if ($cacheOptions) {
            $cacheDir   = '/tmp/liveuser/';
            $lifeTime   = 3600;
            $cacheName  = 'LIVEUSER_CONFIG';
            $force      = false;
            if (is_array($cacheOptions)) {
                extract($cacheOptions);
            }
            $cacheOptions = array(
                'cacheDir' => $cacheDir,
                'lifeTime' => $lifeTime
            );
            if (!$force) {
                @include_once 'Cache/Lite.php';
                $clite = new Cache_Lite($cacheOptions);
                if ($data = $clite->get($cacheName)) {
                    $data = explode('|~|',$data);

                    $this->authContainers   = unserialize($data[0]);
                    $this->permContainer    = unserialize($data[1]);
                    $t = $data[2]          != '' ? unserialize($data[2]) : null;
                    if (is_array($t)) {
                        foreach ($t as $k=>$v) {
                            $this->$k = $v;
                        }
                    }
                    return true;
                }
            }
        }

        @include_once 'XML/Tree.php';
        if (class_exists('XML_Tree')) {
            $tree =& new XML_Tree($configFile);
            $t = &$tree->getTreeFromFile();

            foreach ($t->children as $node) {
                switch ($node->name) {
                    case 'authContainers':
                        foreach ($node->children as $container) {
                            foreach ($container->children as $options) {
                                $this->authContainers[] = array_merge(
                                        $options->attributes,
                                        $container->attributes
                                    );
                            }
                        }
                        break;
                    case 'permContainer':
                        $this->permContainer = $node->attributes;
                        break;
                    default:
                        if (isset($this->{$node->name})) {
                            $tmp[$node->name]=
                                $this->{$node->name} = $node->content;
                        }
                }
            }

            if ($cacheOptions) {
                /*
                 * |~| is a weird separator, we may not find it in
                 * the data :)
                 */
                $cache = serialize($this->authContainers).'|~|'.
                         serialize($this->permContainer).'|~|'.
                         serialize($tmp);
                $clite->save($cache);
            }

            unset($t, $tree, $options);
            return true;
        } else {
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessage(LIVEUSER_MISSING_DEPS),
                LIVEUSER_MISSING_DEPS
            );
        }
     }

    /**
     * LiveUser::readConfigArray()
     *
     * Reads a configuration array specified in $configArray
     * and sets the instance vars to the values found.
     *
     * @param  string   $configArray
     * @return boolean
     * @access public
     */
    function readConfigArray($configArray)
    {
        if (is_array($configArray)) {
            foreach ($configArray as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            return true;
        }

        $this->_error = true;

        return LiveUser::raiseError(
            LiveUser::errorMessage(LIVEUSER_CONFIG_ARRAY_NOT_AN_ARRAY),
            LIVEUSER_CONFIG_ARRAY_NOT_AN_ARRAY
        );
    }

    /**
     * LiveUser::checkRight()
     *
     * Wrapper method for the permission object's own checkRight method.
     *
     * @param  integer  $rights     A right name or an array of rights.
     * @param  integer  $area_id    Id of the area to check.
     * @return boolean
     * @access public
     */
    function checkRight($rights)
    {
        if ($this->_error) {
            return false;
        }
        if (is_object($this->_perm)) {
            $hasright = false;

            if (is_array($rights)) {
                foreach($rights as $currentright) {
                    if ($this->checkRight($currentright)) {
                        $hasright = true;
                    } else {
                        $hasright = false;
                        break;
                    }
                }
            } else {
                // Remember: $rights is a single value at this point!
                $hasright = $this->_perm->checkRight($rights);
            }

            return $hasright;
        } else {
            return false;
        }
    }

    /**
     * LiveUser::checkRightLevel()
     *
     * Wrapper method for the permission object's own checkRightLevel method.
     *
     * @param  integer  $right_id
     * @param  integer  $owner_user_id
     * @param  integer  $owner_group_id
     * @param  integer  $area_id
     * @return boolean
     * @access public
     */
    function checkRightLevel($right_id, $owner_user_id, $owner_group_id)
    {
        if ($this->_error) {
            return false;
        }
        if (is_object($this->_perm)) {
            return ($this->_perm->checkRightLevel($right_id, $owner_user_id, $owner_group_id));
        } else {
            return false;
        }
    }

    /**
     * LiveUser::isLoggedIn()
     *
     * Checks if a user is logged in.
     *
     * @return boolean
     * @access public
     */
    function isLoggedIn()
    {
        if ($this->_error) {
            return false;
        }
        if (!is_object($this->_auth)) {
            return false;
        } elseif ($this->_auth->loggedIn == true) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * LiveUser::isInactive()
     *
     * Function that determines if the user exists but hasn't yet been declared
     * "active" by an administrator. Use this to check if this was the reason
     * why a user was not able to login.
     * true == Not active
     * false == active
     *
     * @return boolean
     * @access public
     */
    function isInactive()
    {
        if ($this->_error) {
            return false;
        }
        if (is_object($this->_auth)
            && $this->_auth->isActive == 0
            && $this->_auth->handle != '') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * LiveUser::setLoginFunction()
     *
     * The user can set a function that will be called if the user
     * tries to access a page wihout logging in first. It will receive
     * the liveuser object. If an empty string or a non-existent function is passed
     * it deactivates the call.
     *
     * @access  public
     * @param   string  $functionName  The name of the function to be called.
     */
    function setLoginFunction($functionName)
    {
        if ($this->_error) {
            return false;
        }
        if (!empty($functionName) && function_exists($functionName)) {
            $this->loginFunction = $functionName;
        } else {
            return LiveUser::raiseError(
                LiveUser::errorMessage(LIVEUSER_MISSING_LOGINFUNCTION),
                LIVEUSER_MISSING_LOGINFUNCTION
            );
        }
    }

    /**
     * LiveUser::setLogoutFunction()
     *
     * The user can set a function that will be called if the user
     * wants to logout (by providing the appropriate GET-parameter, see
     * property $logoutParamName).
     * If an empty string or a non-existent function is passed
     * it deactivates the call.
     *
     * @access  public
     * @param   string  $functionName  The name of the function to be called.
     */
    function setLogoutFunction($functionName)
    {
        if ($this->_error) {
            return false;
        }
        if (!empty($functionName) && function_exists($functionName)) {
            $this->logoutFunction = $functionName;
        } else {
            return LiveUser::raiseError(
                LiveUser::errorMessage(LIVEUSER_MISSING_LOGOUTFUNCTION),
                LIVEUSER_MISSING_LOGOUTFUNCTION
            );
        }
    }


    /**
     * LiveUser::getHandle()
     *
     * Wrapper method for the auth object's own getHandle method.
     *
     * @access public
     * @return string   The handle of the current user.
     *                  Returns an empty string if no user logged.
     */
    function getHandle()
    {
        return !$this->_error && $this->_auth ?
                $this->_auth->getProperty('handle') : '';
    }

    /**
     * LiveUser::getProperty()
     *
     * Wrapper method to access properties from the auth and
     * permission containers.
     *
     * @access public
     * @param  string   Name of the property to be returned.
     * @return mixed    NULL, a value or an array.
     */
    function getProperty($what)
    {
        if ($this->_error) {
            return false;
        }
        $that = null;
        if ($this->_auth && $this->_auth->getProperty($what) !== null) {
            $that = $this->_auth ? $this->_auth->getProperty($what) : null;
        } elseif ($this->_perm && $this->_perm->getProperty($what) !== null) {
            $that = $this->_perm ? $this->_perm->getProperty($what) : null;
        }
        return $that;
    }

    /**
     * LiveUser::getRights()
     *
     * Returns a one-dimensional array with all rights assigned
     * to this user. Array format depends on the optional parameter:
     * TRUE: array(intRight_ID => intRightLevel, ...)
     * FALSE array(intRight_ID, ...) [Default]
     * If no rights are available, FALSE is returned.
     *
     * @param  boolean   $withLevels   Return array with right_ids as key and level as value
     * @access public
     * @return mixed
     */
    function getRights($withLevels = false)
    {
    	if (is_object($this->_perm)) {
        	return($this->_perm->getRights($withLevels));
    	}
    	return(false);
    }

    /**
     * Get the current status
     *
     * @access public
     * @return integer
     */
    function getStatus()
    {
        return $this->status;
    }

    /**
     * Generates a md5 from given password and stores it on the server
     * along with crypted password.
     *
     * Cookies are not secure but keeping
     * the password in plain text in the cookie
     * is not the best way to go. Since some
     * containers like LDAP need the clear text
     * password, we store an encrypted version
     * of the password using Crypt_Rc4 which provides
     * a simple two-way mechanism.
     *
     * To do this LiveUser needs access
     * to a writeable directory. If you do
     * no have access to the ini_get()
     * function please set a constant
     * named LIVEUSER_TMPDIR with an absolute
     * path to a writeable directory.
     *
     * @access private
     * @param  string   $passwd  the password to store
     * @return boolean  true if success, false otherwise
     */
    function _storeCookiePasswdId($passwd, $store)
    {
        if (!defined('LIVEUSER_TMPDIR')) {
            define('LIVEUSER_TMPDIR', ini_get('session.save_path'));
        }

        if (!$fh = fopen(LIVEUSER_TMPDIR . "/$store.lu", 'w')) {
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessages(LIVEUSER_CANNOT_ACCESS_COOKIE_STORE),
                LIVEUSER_CANNOT_ACCESS_COOKIE_STORE,
                'Cannot open file in write mode');
        }

        $data = serialize(array(md5($passwd), $passwd));

        if (PEAR::loadExtension('mcrypt')) {
            $td = mcrypt_module_open ('tripledes', '', 'ecb', '');
            $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size ($td), MCRYPT_RAND);
            mcrypt_generic_init($td, $this->_cookieEncryptionKey, $iv);
            $crypted_data = mcrypt_generic($td, $data);
            mcrypt_generic_deinit($td);
            mcrypt_module_close($td);
        } else {
            include_once 'Crypt/Rc4.php';

            $crypt =& new Crypt_RC4($this->_cookieEncryptionKey);
            $crypted_data = $crypt->crypt($data);
        }

        if (!fwrite($fh, $crypted_data)) {
            fclose($fh);
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessages(LIVEUSER_CANNOT_ACCESS_COOKIE_STORE),
                LIVEUSER_CANNOT_ACCESS_COOKIE_STORE,
                'Cannot write to file ' . $store . '.lu');
        }

        fclose($fh);

        return true;
    }

    /**
     * Reads a "store" on the server in which the password
     * and password id are stored encrypted.
     *
     * @access private
     * @param  string $store  the filename of the store
     * @return mixed  an array of the data, false otherwise
     */
    function _readStoreCookiePasswdId($store)
    {
        if (!defined('LIVEUSER_TMPDIR')) {
            define('LIVEUSER_TMPDIR', ini_get('session.save_path'));
        }

        if (!$fh = fopen(LIVEUSER_TMPDIR . "/$store.lu", 'r')) {
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessages(LIVEUSER_CANNOT_ACCESS_COOKIE_STORE),
                LIVEUSER_CANNOT_ACCESS_COOKIE_STORE,
                'Cannot open file in read mode');
        }

        if (!$fields = fread($fh, 4096)) {
            fclose($fh);
            $this->_error = true;
            return LiveUser::raiseError(
                LiveUser::errorMessages(LIVEUSER_CANNOT_ACCESS_COOKIE_STORE),
                LIVEUSER_CANNOT_ACCESS_COOKIE_STORE,
                'Cannot read data from file ' . $store . '.lu');
        }

        fclose($fh);

        if (function_exists('mcrypt_module_open')) {
            $td = mcrypt_module_open('tripledes', '', 'ecb', '');
            $iv = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_RAND);
            mcrypt_generic_init($td, $this->_cookieEncryptionKey, $iv);
            $crypted_data = mdecrypt_generic($td, $this->_cookieEncryptionKey, $iv);
            mcrypt_generic_deinit($td);
            mcrypt_module_close($td);
        } else {
            include_once 'Crypt/Rc4.php';

            $crypt  =& new Crypt_RC4($this->_cookieEncryptionKey);
            $params = unserialize($crypt->decrypt($fields));
        }

        return (count($params) == 2) ? $params : false;
    }
}
?>