<?php
// LiveUser: A framework for authentication and authorization in PHP applications
// Copyright (C) 2002-2003 Markus Wolff
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
 * DB_Complex container for permission handling
 *
 * @package  LiveUser
 * @category authentication
 */

/**
 * Require parent class definition and PEAR::DB class.
 */
require_once 'LiveUser/Perm/Common.php';
require_once 'DB.php';

/**
 * Class LiveUser_Perm_Container_DB_Complex
 *
 * Description:
 * This is a PEAR::DB backend driver for the LiveUser class.
 * A PEAR::DB connection object can be passed to the constructor to reuse an
 * existing connection. Alternatively, a DSN can be passed to open a new one.
 *
 * Requirements:
 * - File "Liveuser.php" (contains the parent class "LiveUser")
 * - Array of connection options or a PEAR::DB connection object must be
 *   passed to the constructor.
 *   Example: array('dsn' => 'mysql://user:pass@host/db_name')
 *              OR
 *            &$conn (PEAR::DB connection object)
 *
 * @author  Lukas Smith <smith@backendmedia.com>
 * @author  Bjoern Kraus <krausbn@php.net>
 * @version $Id: DB_Complex.php,v 1.27 2003/04/20 19:46:18 arnaud Exp $
 * @package LiveUser
 * @category authentication
 */
class LiveUser_Perm_Container_DB_Complex extends LiveUser_Perm_Common
{
    /**
     * The DSN that was used to connect to the database (set only if no
     * existing connection object has been reused).
     *
     * @var string
     * @access private
     */
    var $dsn = NULL;

    /**
     * PEAR::DB connection object.
     *
     * @var object
     * @access private
     */
    var $dbc = NULL;

    /**
     * Table prefix
     * Prefix for all db tables the container has.
     * @var string
     * @access public
     */
    var $prefix = 'liveuser_';

    /**
     * Indicates if backend module initialized correctly. If yes,
     * TRUE, if not false. Backend module won't initialize if the
     * init value (usually an object or resource handle that
     * identifies the backend to be used) is not of the required
     * type.
     *
     * @var boolean
     * @access public
     */
    var $init_ok = false;

    /**
     * LiveUser_Perm_Container_DB_Complex::LiveUser_Perm_Container_DB_Complex()
     *
     * Class constructor.
     *
     * @param  mixed    $connectOptions  Array or PEAR::DB object.
     * @return void
     */
    function &LiveUser_Perm_Container_DB_Complex(&$connectOptions)
    {
        if (DB::isConnection($connectOptions)) {
            $this->dbc = &$connectOptions;
            $this->init_ok = true;
        } elseif (is_array($connectOptions)) {
            foreach ($connectOptions as $key => $value) {
                if (isset($this->$key)) {
                    $this->$key = $value;
                }
            }
            $this->dbc = DB::connect($connectOptions['dsn']);

            if (!DB::isError($this->dbc)) {
                $this->init_ok = true;
            }
        }
    }

    /**
     * LiveUser_Perm_Container_DB_Complex::init()
     *
     * Tries to find the user with the given user ID in the permissions
     * container. Will read all permission data and return TRUE on success.
     *
     * @access public
     * @param  string   $uid  User id in the auth container.
     * @return boolean  TRUE if init process was successfull else false.
     */
    function init($uid)
    {
        $success = TRUE;
        $query = '
            SELECT
                user_id,
                type
            FROM
                '.$this->prefix.'perm_users
            WHERE
                auth_user_id='.$this->dbc->quote($uid);
        $result = $this->dbc->getRow($query, DB_FETCHMODE_ASSOC);
        if (DB::isError($result)) {
            $success = $result;
        } else {
            $this->permUserId = $result['user_id'];
            $this->userType   = $result['type'];

            if ($this->userType == LIVEUSER_AREAADMIN_TYPE_ID) {
                // get all areas in which the user is area admin
                $query = '
                    SELECT
                        LU.area_id
                    FROM
                        '.$this->prefix.'area_admin_areas LU
                    WHERE
                        LU.user_id='.$this->permUserId;
                $result = $this->dbc->getCol($query);

                if (DB::isError($result)) {
                   return $result;
                }

                $this->areaAdmin = $result;
            }

            $this->readRights();
        }

        return $success;
    } // end func init

    /**
     * LiveUser_Perm_Container_DB_Complex::unfreeze()
     *
     * Reinitializes database connection after the object had been
     * retrieved from the session
     *
     * @param  mixed    $connectOptions  Array or PEAR::DB object.
     * @access public
     * @return void
     */
    function unfreeze(&$connectOptions) {
        // Call constructor again to setup the proper connection
        $this->LiveUser_Perm_Container_DB_Complex($connectOptions);
    }

    /**
     * LiveUser_Perm_Container_DB_Complex::freeze()
     *
     * Prepares the object for being stored in a session (properly
     * disconnect from the database, free up resources)
     *
     * @access public
     * @return void
     */
    function freeze() {
        // Open new connection
        if ($this->dsn) {
            $this->dbc->disconnect();
        }
        $this->dbc = NULL;
    }

    /**
     * LiveUser_Perm_Container_DB_Complex::readUserRights()
     *
     * Reads all individual rights of current user into
     * an array of this format:
     * RightName -> Value
     *
     * @access private
     * @see    readRights()
     * @return void
     */
    function readUserRights()
    {
        $query = '
            SELECT
                R.right_id,
                U.right_level,
                R.has_implied
            FROM
                '.$this->prefix.'rights AS R
            INNER JOIN
                '.$this->prefix.'userrights AS U
            ON
                R.right_id=U.right_id
            WHERE
                U.user_id='.$this->permUserId.'
            ORDER BY
                U.right_level';
        $result = $this->dbc->query($query, DB_FETCHMODE_ASSOC);

        if (!DB::isError($result)) {
            $allRights = array();

            while ($rdata = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
                $currentRight   = $rdata['right_id'];
                $currentLevel   = $rdata['right_level'];
                $currentImplied = $rdata['has_implied'];
                $allRights[$currentRight] = $currentLevel;
                if ($currentImplied) {
                    $queue = array($currentRight);
                    while (count($queue) > 0) {
                        $currentRight = array_shift($queue);
                        $query = '
                            SELECT
                                RI.implied_right_id,
                                '.$currentLevel.'
                            FROM
                                '.$this->prefix.'right_implied AS RI
                            WHERE
                                RI.right_id='.$currentRight;
                        $result2 = $this->dbc->getAssoc($query);

                        if (DB::isError($result2)) {
                            break;
                        } elseif (is_array($result2)) {
                            foreach ($result2 as $key => $val) {
                                    $impliedRights[$key] = $val;
                            }
                            $rights = array_keys($result2);
                            foreach ($rights as $right) {
                                $query = '
                                    SELECT
                                        has_implied
                                    FROM
                                        '.$this->prefix.'rights
                                    WHERE
                                        right_id='.$right;
                                $implied = $this->dbc->getOne($query);
                                if ($implied) {
                                    $queue[] = $right;
                                }
                            }
                        }
                    }
                }
            }
            if (isset($impliedRights) && is_array($impliedRights)) {
                foreach ($impliedRights as $right => $level) {
                   if (isset($allRights[$right])) {
                       $allRights[$right] =
                       max($allRights[$right],
                           $level);
                   } else {
                       $allRights[$right] = $level;
                   }
                }
            }
            $this->userRights = $allRights;
            $result->free();
        }
    } // end func readUserRights

    /**
     * LiveUser_Perm_Container_DB_Complex::readGroupRights()
     *
     * Reads all individual rights of current user into
     * a two-dimensional array of this format:
     * "GroupName" => "RightName" -> "Level"
     *
     * @access private
     * @see    readRights()
     * @return void
     */
    function readGroupRights()
    {
        $allRights = array();

        if (is_array($this->groupIds) && count($this->groupIds) > 0) {
            $query = '
                SELECT
                    DISTINCT GU.group_id,
                    R.right_id,
                    GR.right_level,
                    R.has_implied
                FROM
                    '.$this->prefix.'groupusers AS GU
                INNER JOIN
                    '.$this->prefix.'grouprights AS GR
                ON
                    GU.group_id=GR.group_id
                INNER JOIN
                    '.$this->prefix.'rights AS R
                ON
                    GR.right_id=R.right_id
                WHERE
                    GU.group_id IN ('.implode(', ', $this->groupIds).')
                ORDER BY
                    GR.right_level ASC';
            $result = $this->dbc->query($query, DB_FETCHMODE_ASSOC);

            if (!DB::isError($result)) {
                while($grdata = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
                    $currentGroup   = $grdata['group_id'];
                    $currentRight   = $grdata['right_id'];
                    $currentLevel   = $grdata['right_level'];
                    $currentImplied = $grdata['has_implied'];
                    $allRights[$currentGroup][$currentRight] = $currentLevel;
                    if ($currentImplied) {
                        $queue = array($currentRight);
                        while (count($queue) > 0) {
                            $currentRight = array_shift($queue);
                            $query  = '
                                SELECT
                                    RI.implied_right_id,
                                    '.$currentLevel.'
                                FROM
                                    '.$this->prefix.'right_implied AS RI
                                WHERE
                                    RI.right_id='.$currentRight;
                            $result2 = $this->dbc->getAssoc($query);

                            if (DB::isError($result2)) {
                                break;
                            } elseif( count($result2)>0 ) {
                                foreach ($result2 as $key => $val) {
                                    $impliedRights[$currentGroup][$key] = $val;
                                }
                                $rights = array_keys($result2);
                                foreach($rights as $right) {
                                    $query = '
                                        SELECT
                                            has_implied
                                        FROM
                                            '.$this->prefix.'rights
                                        WHERE
                                            right_id='.$right;
                                    $implied = $this->dbc->getOne($query);
                                    if ($implied) {
                                        $queue[] = $right;
                                    }
                                }
                            }
                        }
                    }
                }
                if (isset($impliedRights) && is_array($impliedRights)) {
                    foreach ($impliedRights as $currentGroup => $groupRright) {
                       foreach ($groupRright as $right => $level) {
                           if (isset($allRights[$currentGroup][$right])) {
                               $allRights[$currentGroup][$right] =
                               max($allRights[$currentGroup][$right], $level);
                           } else {
                               $allRights[$currentGroup][$right] = $level;
                           }
                       }
                    }
                }
                $result->free();
            }
        }
        $this->groupRights = $allRights;
    } // end func readGroupRights

    /**
     * LiveUser_Perm_Container_DB_Complex::readGroups()
     *
     * Reads all the group ids in that the user is also a member of
     * (all groups that are subgroups of these are also added recursively) and
     * reads all user ids that are a memember of these (sub)groups
     *
     * @access private
     * @see    readRights()
     * @return void
     */
    function readGroups()
    {
        $query = '
            SELECT
                GU.group_id
            FROM
                '.$this->prefix.'groupusers AS GU
            INNER JOIN
                '.$this->prefix.'groups AS G
            ON
                GU.group_id=G.group_id
            WHERE
                G.is_active='.$this->dbc->quote('Y').'
            AND
                user_id='.$this->permUserId;
        $result = $this->dbc->getCol($query);

        if (!DB::isError($result)) {
            $this->groupIds = $result;
        } else {
            $this->groupIds = array();
        }

        // get all subgroups recursively
        // WARNING: this will explode if there are any evil loops
        // in the group-subgroup structure
        while (count($result) > 0) {
            $query = '
                SELECT
                    DISTINCT subgroup_id
                FROM
                    '.$this->prefix.'group_subgroups
                WHERE
                    group_id IN ('.implode(', ', $result).')
                    AND subgroup_id NOT IN ('.implode(', ', $this->groupIds).')';
            $result = $this->dbc->getCol($query);

            if (DB::isError($result)) {
                break;
            } else {
                $this->groupIds = array_merge($result, $this->groupIds);
            }
        }
        if (count($this->groupIds) > 0) {
            $query = '
                SELECT
                    DISTINCT user_id
                FROM
                    '.$this->prefix.'groupusers
                WHERE
                    group_id IN ('.implode(', ', $this->groupIds).')';
            $result = $this->dbc->getCol($query);

            if (!DB::isError($result)) {
                $this->groupUserIds = $result;
            }
        }
    } // end func readGroups

    /**
     * LiveUser_Perm_Container_DB_Complex::readRights()
     *
     * Reads all rights of current user into an array.
     * Group rights and invididual rights are being merged
     * in the process.
     *
     * @access private
     * @return void
     */
    function readRights()
    {
        $this->readUserRights();
        $this->readGroups();
        $this->readGroupRights();

        // Flatten group rights
        if (is_array($this->groupRights)) {
            foreach ($this->groupRights as $currentGroup => $rights) {
               foreach ($rights as $right => $level) {
                   $tmpRights[$right] = $level;
               }
            }
        } else {
            $this->groupRights = false;
        }

        // Check if user has individual rights...
        if (is_array($this->userRights)) {
            // Overwrite values from temporary array with values from userrights
            foreach ($this->userRights as $right => $level) {
               if (isset($tmpRights[$right])) {
                   if ($level < 0) {
                       $tmpRights[$right] = $tmpRights[$right] - $level;
                   } else {
                       $tmpRights[$right] = max($tmpRights[$right], $level);
                   }
               } else {
                   $tmpRights[$right] = $level;
               }
            }
        } else {
            $this->userRights = false;
        }

        // Strip values from array if level is not greater than zero
        if (isset($tmpRights) && is_array($tmpRights)) {
            foreach ($tmpRights as $right => $level) {
               if ($level > 0) {
                   $cRights[$right] = $level;
               }
            }
        }

        if (isset($cRights) && is_array($cRights)) {
            $this->rights = $cRights;
        } else {
            $this->rights = array();
        }
    } // end func readRights

    /**
     * LiveUser_Perm_Container_DB_Complex::checkLevel()
     *
     * Checks if the current user has a certain right in a
     * given area at the necessary level.
     *
     * Level 1: requires that owner_user_id matches $this->permUserId
     * Level 2: requires that the $owner_group_id matches the id one of
     *          the (sub)groups that $this->permUserId is a memember of
     *          or requires that the $owner_user_id matches a user_id of
     *          a memeber of one of $this->permUserId's (sub)groups
     * Level 3: no requirements
     *
     * Important note:
     *          Every ressource MAY be owned by a user and/or by a group.
     *          Therefore, $owner_user_id and/or $owner_group_id can
     *          either be an integer or NULL.
     *
     * @access private
     * @see    checkRightLevel()
     * @param  integer  $level          Level value as returned by checkRight().
     * @param  integer  $owner_user_id  Id of the owner of the ressource for
     *                                  which the right is requested.
     * @param  integer  $owner_group_id Id of the group of the ressource for
     *                                  which the right is requested.
     * @return boolean  true if the level is sufficient to grant access else false.
     */
    function checkLevel($level, $owner_user_id, $owner_group_id)
    {
        $hasright = false;

        // highest level (that is level 3)
        if ($level == LIVEUSER_MAX_LEVEL) {
            $hasright = true;
        } elseif ($level >= 1) {
            // level 1 or higher
            if (is_numeric($owner_user_id) && $this->permUserId == $owner_user_id) {
               $hasright = true;
            // level 2 or higher
            } elseif ($level >= 2) {
                // check if the ressource is owned by a (sub)group that the user is part of
                // or if the ressource is owned by a member of a (sub)group of the user
                // not sure if this feature is worth the performance cost
                // as this would need a possibly very large array of user id's
                if (is_numeric($owner_group_id) && in_array($owner_group_id, $this->groupIds)) {
                    $hasright = true;
                }
            }
        }
        return $hasright;
    } // end func checkLevel

    /**
     * LiveUser_Perm_Container_DB_Complex::checkRightLevel()
     *
     * Checks if the current user has a certain right in a
     * given area with the sufficient level to gain access
     * to a specific ressource in that area.
     *
     * @access public
     * @see    LiveUser_Perm_Common:checkRight(), checkLevel()
     * @param  integer  $right_id        Id of the right within the AuthArea to check for.
     * @param  integer  $owner_user_id   Id of the owner of the ressource for
     *                                   which the right is requested.
     * @param  integer  $owner_group_id  Id of the group of the ressource for
     *                                   which the right is requested.
     * @param  integer  $area_id         Id of the AuthArea to use.
     * @return boolean  true if the level is sufficient to grant access else false.
     */
    function checkRightLevel($right_id, $owner_user_id, $owner_group_id, $area_id)
    {
        $level = $this->checkRight($right_id, $area_id);
        $hasright = $this->checkLevel($level, $owner_user_id, $owner_group_id);

        return $hasright;
    }

    /**
     * LiveUser_Perm_Container_DB_Complex::userExists()
     *
     * Checks if a user with the given user_id exists in the
     * permission container and returns true on success.
     *
     * @access public
     * @param  integer  $user_id  The users id in the permission table.
     * @return boolean  true if the id was found, else false.
     */
    function userExists($user_id)
    {
        if ($this->init_ok) {
            $query = '
                SELECT
                    1
                FROM
                    '.$this->prefix.'perm_users
                WHERE
                    user_id='.$user_id;
            return $this->dbc->getOne($query);
        } else {
            return false;
        }
    }
}
?>
